package br.com.mss.labqp.admin.dao;

import java.util.List;
import java.util.Map;

import org.nextframework.authorization.AuthorizationDAO;
import org.nextframework.authorization.Permission;
import org.nextframework.authorization.Role;
import org.nextframework.authorization.User;
import org.nextframework.persistence.QueryBuilder;
import org.springframework.orm.hibernate3.HibernateTemplate;

import br.com.mss.labqp.admin.entity.Colaborador;
import br.com.mss.labqp.admin.entity.ColaboradorPapel;
import br.com.mss.labqp.admin.entity.Papel;
import br.com.mss.labqp.admin.entity.Permissao;

public class NextAuthorizationDAO implements AuthorizationDAO  {

	private HibernateTemplate hibernateTemplate;

	public void setHibernateTemplate(HibernateTemplate hibernateTemplate) {
		this.hibernateTemplate = hibernateTemplate;
	}

	// API
	/**
	 * Retorna a lista de todos os papeis do sistema
	 */
	public Role[] findAllRoles() {
		List<Role> roles = new QueryBuilder<Role>().from(
				Papel.class).list();
		return roles.toArray(new Role[roles.size()]);
	}

	// API
	/**
	 * Acha a permisso para determinado nvel em determinada tela
	 * 
	 * @param role
	 *            papel que desejamos a permisso
	 * @param controlName
	 *            path que representa a tela desejada
	 */
	public Permission findPermission(Role role, String controlName) {
		return new QueryBuilder<Permission>()
				.from(Permissao.class).where("permissao.role = ?", role)
				.where("permissao.path = ?", controlName).unique();
	}

	// API
	/**
	 * Acha o usurio pelo login
	 */
	public User findUserByLogin(String login) {
		System.out.println("findUserByLogin->"+login);
		return new QueryBuilder<User>()
				.from(Colaborador.class).where("colaborador.login = ?", login)
				.unique();
	}

	// API
	/**
	 * Acha os papeis (nveis) de determinado usurio
	 */
	public Role[] findUserRoles(User user) {
		List<Role> list = new QueryBuilder<Role>()
				.select("papel").from(ColaboradorPapel.class)
				.leftOuterJoin("colaboradorPapel.papel papel")
				.leftOuterJoin("colaboradorPapel.colaborador colaborador")
				.where("colaborador = ?", user).list();
		return list.toArray(new Role[list.size()]);
	}

	// API
	/**
	 * Salva ou atualiza uma permisso no banco de dados.
	 */
	public Permission savePermission(String controlName, Role role,
			Map<String, String> permissionMap) {
		Permissao permissao = (Permissao) findPermission(role, controlName);
		System.out.println("PATH->"+controlName );
		if (permissao == null) {
			permissao = new Permissao();
			permissao.setPath(controlName);
			permissao.setRole((Papel) role);
		}
		permissao.setPermissionMap(permissionMap);
		hibernateTemplate.saveOrUpdate(permissao);
		return permissao;
	}

	@Override
	public long getLastUpdateTime(Role role) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public long getLastUpdateTime() {
		// TODO Auto-generated method stub
		return 0;
	}

}