package br.com.mss.labqp.cadastro.dao;

import java.util.List;

import org.nextframework.controller.crud.FiltroListagem;
import org.nextframework.persistence.DefaultOrderBy;
import org.nextframework.persistence.QueryBuilder;
import org.nextframework.persistence.SaveOrUpdateStrategy;

import br.com.mss.labqp.GenericAuditDAO;
import br.com.mss.labqp.cadastro.entity.Cliente;
import br.com.mss.labqp.cadastro.entity.ClienteFiltro;
import br.com.mss.labqp.geral.FiltroExcluidoEnum;
import br.com.mss.labqp.geral.TipoCadastroEnum;
import br.com.mss.labqp.geral.TipoPessoaEnum;

@DefaultOrderBy("cliente.nome")
public class ClienteDAO extends GenericAuditDAO<Cliente> {

	@Override
	public void updateListagemQuery(QueryBuilder<Cliente> query,
			FiltroListagem _filtro) {
		ClienteFiltro filtro = (ClienteFiltro) _filtro;
		query.leftOuterJoinFetch("cliente.foto");
		query.whereLike("cliente.nome", filtro.getNome());
		query.whereLike("cliente.nomeFantasia", filtro.getNomeFantasia());
		query.whereLike("cliente.pontoReferencia", filtro.getPontoReferencia());
		if (filtro.getTipoPessoa() == TipoPessoaEnum.FISICA) {
			query.where("cliente.cpf = ?", filtro.getCpf());
		} else if (filtro.getTipoPessoa() == TipoPessoaEnum.JURIDICA) {
			query.where("cliente.cnpj = ?", filtro.getCnpj());
		}
		if (filtro.getTipoCadastro() != null)
			query.where("cliente.tipoCadastro = "
					+ filtro.getTipoCadastro().ordinal());
		if (filtro.getTipoPessoa() != null)
			query.where("cliente.tipoPessoa = "
					+ filtro.getTipoPessoa().ordinal());
		if (filtro.getFiltroExcluido() != null) {
			if (filtro.getFiltroExcluido() == FiltroExcluidoEnum.SIM)
				query.where("cliente.dataExc is not null");
			else
				query.where("cliente.dataExc is null");

		}
	}

	public Boolean findByTipoPessoa(TipoPessoaEnum tipoPessoa, String cpf_cnpj) {
		QueryBuilder<Cliente> query = query();
		if (tipoPessoa == TipoPessoaEnum.FISICA) {
			query.where("cliente.cpf = '"+  cpf_cnpj + "'");
		} else if (tipoPessoa == TipoPessoaEnum.JURIDICA) {
			query.where("cliente.cnpj = '"+  cpf_cnpj + "'");
		}
		query.where("cliente.dataExc is null");
 		List<Cliente> clientes = query.list();
 		
		return (clientes != null && clientes.size() > 0);
	}
	
	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateSaveOrUpdate(org.nextframework.persistence.SaveOrUpdateStrategy)
	 */
	@Override
	public void updateSaveOrUpdate(SaveOrUpdateStrategy save) {
		save.saveOrUpdateManaged("contatos");
		save.saveOrUpdateManaged("agendas");
		save.saveOrUpdateManaged("enderecos");
		super.updateSaveOrUpdate(save);
	}

	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateEntradaQuery(org.nextframework.persistence.QueryBuilder)
	 */
	@Override
	public void updateEntradaQuery(QueryBuilder<Cliente> query) {
		query.fetchCollection("contatos");
		query.fetchCollection("agendas");
		query.fetchCollection("enderecos");
		super.updateEntradaQuery(query);
	}

	/* (non-Javadoc)
	 * @see br.com.mss.labqp.GenericAuditDAO#getQueryFindForCombo(java.lang.String)
	 */
	@Override
	protected String getQueryFindForCombo(String hbQueryFindForCombo) {
		// TODO Auto-generated method stub
		hbQueryFindForCombo = super.getQueryFindForCombo(hbQueryFindForCombo);
		hbQueryFindForCombo = hbQueryFindForCombo.concat(" and ( cliente.tipoCadastro = " );
		hbQueryFindForCombo = hbQueryFindForCombo.concat( String.valueOf(TipoCadastroEnum.CLIENTE.ordinal()) );
		hbQueryFindForCombo = hbQueryFindForCombo.concat(" ) " );
		return hbQueryFindForCombo;
	}

}
