package br.com.mss.labqp.cadastro.dao;

import org.nextframework.controller.crud.FiltroListagem;
import org.nextframework.persistence.DefaultOrderBy;
import org.nextframework.persistence.QueryBuilder;

import br.com.mss.labqp.GenericAuditDAO;
import br.com.mss.labqp.cadastro.entity.Cliente;
import br.com.mss.labqp.cadastro.entity.Inspetoria;
import br.com.mss.labqp.cadastro.entity.InspetoriaFiltro;
import br.com.mss.labqp.geral.FiltroExcluidoEnum;

@DefaultOrderBy("inspetoria.nome")
public class InspetoriaDAO extends GenericAuditDAO<Inspetoria> {
     
	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateListagemQuery(org.nextframework.persistence.QueryBuilder, org.nextframework.controller.crud.FiltroListagem)
	 */
	@Override
	public void updateListagemQuery(QueryBuilder<Inspetoria> query,
			FiltroListagem _filtro) {
		InspetoriaFiltro filtro = (InspetoriaFiltro) _filtro;
		query.whereLike("inspetoria.nome", filtro.getNome());
		if (filtro.getFiltroExcluido() != null) {
			if (filtro.getFiltroExcluido() == FiltroExcluidoEnum.SIM)
				query.where("inspetoria.dataExc is not null");
			else
				query.where("inspetoria.dataExc is null");

		}
	}

	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateEntradaQuery(org.nextframework.persistence.QueryBuilder)
	 */
	@Override
	public void updateEntradaQuery(QueryBuilder<Inspetoria> query) {
		query.fetchCollection("clientes");
		super.updateEntradaQuery(query);
	}

	/* (non-Javadoc)
	 * @see br.com.mss.labqp.GenericAuditDAO#delete(java.lang.Object)
	 * Alterado para os clientes vinculados a inspetoria eliminada informando null no campo de vinculao
	 */
	@Override
	public void delete(Inspetoria bean) {
		Inspetoria inspetoria = this.loadForEntrada(bean);
		if (inspetoria.getClientes() != null && 
				inspetoria.getClientes().size() > 0) {
			for (Cliente cliente : inspetoria.getClientes()) {
				cliente.setInspetoria(null);
			}
			getHibernateTemplate().saveOrUpdateAll(inspetoria.getClientes());
		}
		super.delete(inspetoria);
	}

}
