package br.com.mss.labqp.cadastro.entity;

import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Index;
import org.nextframework.bean.annotation.DisplayName;
import org.nextframework.types.Cnpj;
import org.nextframework.types.Cpf;
import org.nextframework.types.Telefone;
import org.nextframework.validation.annotation.MaxLength;
import org.nextframework.validation.annotation.Required;

import br.com.mss.labqp.Pessoa;
import br.com.mss.labqp.agenda.entity.Agenda;
import br.com.mss.labqp.geral.TipoCadastroEnum;
import br.com.mss.labqp.geral.TipoPessoaEnum;

@Entity
@Table(name = "cliente")
@DisplayName("Cliente/Fornecedor")
@SequenceGenerator(allocationSize = 1, name = "SEQUENCE", sequenceName = "SQ_CLIENTE_ID")
@org.hibernate. annotations.Table (appliesTo = "cliente", 
indexes = {@Index(name = "IDX_CLIENTE_001",columnNames = {"cnpj"}),
		@Index(name = "IDX_CLIENTE_002",columnNames = {"cpf"})})
public class Cliente extends Pessoa {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2876919721769031557L;

	private String nomeFantasia;
	private String inscricaoEstadual;
	private TipoPessoaEnum tipoPessoa;
	private TipoCadastroEnum tipoCadastro;
	private Telefone telefone;
	private Telefone celular;
	private Cnpj cnpj;
	private Cpf cpf;
	private String cispoa;
	private String tipoCadastroStr;
	private Inspetoria inspetoria;
	private Vendedor vendedor;
	private List<EnderecoCliente> enderecos;
	private List<Contato> contatos;
	private List<Agenda> agendas;

	/**
	 * @return the nomeFantasia
	 */
	@MaxLength(50)
	@Required
	@Column(length = 50,unique=true,nullable=false)
	public String getNomeFantasia() {
		return nomeFantasia;
	}

	/**
	 * @param nomeFantasia the nomeFantasia to set
	 */
	public void setNomeFantasia(String nomeFantasia) {
		this.nomeFantasia = nomeFantasia;
	}

	/**
	 * @return the inscricaoEstadual
	 */
	@Column(length = 50)
	@MaxLength(50)
	public String getInscricaoEstadual() {
		return inscricaoEstadual;
	}

	/**
	 * @param inscricaoEstadual the inscricaoEstadual to set
	 */
	public void setInscricaoEstadual(String inscricaoEstadual) {
		this.inscricaoEstadual = inscricaoEstadual;
	}

	@Column(length = 20)
	@MaxLength(20)
	public Telefone getTelefone() {
		return telefone;
	}

	public void setTelefone(Telefone telefone) {
		this.telefone = telefone;
	}

	@Column(length = 20)
	@MaxLength(20)
	public Telefone getCelular() {
		return celular;
	}

	public void setCelular(Telefone celular) {
		this.celular = celular;
	}

	@Column(length = 20)
	@MaxLength(20)
	public Cnpj getCnpj() {
		return cnpj;
	}

	public void setCnpj(Cnpj cnpj) {
		this.cnpj = cnpj;
	}

	@Column(length = 14)
	@MaxLength(14)
	public Cpf getCpf() {
		return cpf;
	}

	public void setCpf(Cpf cpf) {
		this.cpf = cpf;
	}

	/**
	 * @return the tipoPessoa
	 */
	@Enumerated(EnumType.ORDINAL)
	@Required
	@DisplayName("Tipo de Pessoa")
	public TipoPessoaEnum getTipoPessoa() {
		return tipoPessoa;
	}

	/**
	 * @param tipoPessoa
	 *            the tipoPessoa to set
	 */
	public void setTipoPessoa(TipoPessoaEnum tipoPessoa) {
		this.tipoPessoa = tipoPessoa;
	}

	/**
	 * @return the tipoCadastro
	 */
	@Enumerated(EnumType.ORDINAL)
	@Required
	@DisplayName("Tipo de Cadastro")
	public TipoCadastroEnum getTipoCadastro() {
		return tipoCadastro;
	}

	/**
	 * @param tipoCadastro
	 *            the tipoCadastro to set
	 */
	public void setTipoCadastro(TipoCadastroEnum tipoCadastro) {
		this.tipoCadastro = tipoCadastro;
	}

	/**
	 * @return the contatos
	 */
	@OneToMany(mappedBy = "cliente")
	public List<Contato> getContatos() {
		return contatos;
	}

	/**
	 * @param contatos the contatos to set
	 */
	public void setContatos(List<Contato> contatos) {
		this.contatos = contatos;
	}

	/**
	 * @return the cispoa
	 */
	@Column(length = 20)
	@MaxLength(20)
	@DisplayName("CISPOA")
	public String getCispoa() {
		return cispoa;
	}

	/**
	 * @param cispoa the cispoa to set
	 */
	public void setCispoa(String cispoa) {
		this.cispoa = cispoa;
	}

	/**
	 * @return the agendas
	 */
	@OneToMany(mappedBy = "cliente")
	public List<Agenda> getAgendas() {
		return agendas;
	}

	/**
	 * @param agendas the agendas to set
	 */
	public void setAgendas(List<Agenda> agendas) {
		this.agendas = agendas;
	}

	/**
	 * @return the tipoCadastroStr
	 */
	@Transient
	public String getTipoCadastroStr() {
		this.setTipoCadastroStr(this.tipoCadastro.name());
		return tipoCadastroStr;
	}

	/**
	 * @param tipoCadastroStr the tipoCadastroStr to set
	 */
	public void setTipoCadastroStr(String tipoCadastroStr) {
		this.tipoCadastroStr = tipoCadastro.name();
	}

	/**
	 * @return the enderecos
	 */
	@OneToMany(mappedBy = "cliente")
	public List<EnderecoCliente> getEnderecos() {
		return enderecos;
	}

	/**
	 * @param enderecos the enderecos to set
	 */
	public void setEnderecos(List<EnderecoCliente> enderecos) {
		this.enderecos = enderecos;
	}

	/**
	 * @return the inspetoria
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	public Inspetoria getInspetoria() {
		return inspetoria;
	}

	/**
	 * @param inspetoria the inspetoria to set
	 */
	public void setInspetoria(Inspetoria inspetoria) {
		this.inspetoria = inspetoria;
	}

	/**
	 * @return the inspetoria
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	public Vendedor getVendedor() {
		return vendedor;
	}

	/**
	 * @param vendedor the vendedor to set
	 */
	public void setVendedor(Vendedor vendedor) {
		this.vendedor = vendedor;
	}

}
