package br.com.mss.labqp.cadastro.entity;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.nextframework.bean.annotation.DescriptionProperty;
import org.nextframework.bean.annotation.DisplayName;
import org.nextframework.types.Telefone;
import org.nextframework.validation.annotation.Email;
import org.nextframework.validation.annotation.MaxLength;
import org.nextframework.validation.annotation.Required;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.format.annotation.DateTimeFormat.ISO;

import br.com.mss.labqp.BasicBean;
import br.com.mss.labqp.geral.TipoEnderecoEnum;

@Entity
@Table(name = "contato")
@SequenceGenerator(allocationSize = 1, name = "SEQUENCE", sequenceName = "SQ_CONTATO_ID")
@DisplayName("Contatos")
public class Contato extends BasicBean {

	/**
	 * 
	 */
	private static final long serialVersionUID = 170781901451599322L;
	private Cliente cliente;
	private String nome;
	private String cargo;
	private Telefone telefone;
	private String email;
	private Date dtNasc;
	private TipoEnderecoEnum tipoContato;
	private Boolean emailLaudo;

	@ManyToOne(fetch = FetchType.LAZY)
	@Required
	public Cliente getCliente() {
		return cliente;
	}

	public void setCliente(Cliente cliente) {
		this.cliente = cliente;
	}

	/**
	 * @return the nome
	 */
	@DescriptionProperty
	@Required
	@Column(length = 50)
	@MaxLength(50)
	public String getNome() {
		return nome;
	}

	/**
	 * @param nome the nome to set
	 */
	public void setNome(String nome) {
		this.nome = nome;
	}

	/**
	 * @return the cargo
	 */
	@Column(length = 20)
	@MaxLength(20)
	public String getCargo() {
		return cargo;
	}

	/**
	 * @param cargo the cargo to set
	 */
	public void setCargo(String cargo) {
		this.cargo = cargo;
	}

	/**
	 * @return the telefone
	 */
	@Column(length = 20)
	@MaxLength(20)
	public Telefone getTelefone() {
		return telefone;
	}

	/**
	 * @param telefone the telefone to set
	 */
	public void setTelefone(Telefone telefone) {
		this.telefone = telefone;
	}

	/**
	 * @return the email
	 */
	@Column(length = 50)
	@MaxLength(50)
	@Email
	public String getEmail() {
		return email;
	}

	/**
	 * @param email the email to set
	 */
	public void setEmail(String email) {
		this.email = email;
	}

	/**
	 * @return the dtNasc
	 */
	@DateTimeFormat(iso=ISO.DATE,pattern="dd/MM/yyyy")
	@Temporal(TemporalType.DATE)
	@DisplayName("Data Nasc.")
	public Date getDtNasc() {
		return dtNasc;
	}

	/**
	 * @param dtNasc the dtNasc to set
	 */
	public void setDtNasc(Date dtNasc) {
		this.dtNasc = dtNasc;
	}

	/**
	 * @return the emailLaudo
	 */
	public Boolean getEmailLaudo() {
		return emailLaudo;
	}

	/**
	 * @param emailLaudo the emailLaudo to set
	 */
	public void setEmailLaudo(Boolean emailLaudo) {
		this.emailLaudo = emailLaudo;
	}

	/**
	 * @return the tipoContato
	 */
	public TipoEnderecoEnum getTipoContato() {
		return tipoContato;
	}

	/**
	 * @param tipoContato the tipoContato to set
	 */
	public void setTipoContato(TipoEnderecoEnum tipoContato) {
		this.tipoContato = tipoContato;
	}

}
