package br.com.mss.labqp.cadastro.entity;

import java.math.BigDecimal;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import org.nextframework.bean.annotation.DisplayName;
import org.nextframework.types.Money;
import org.nextframework.validation.annotation.MaxLength;
import org.nextframework.validation.annotation.MaxValue;
import org.nextframework.validation.annotation.Required;
import org.springframework.format.annotation.NumberFormat;
import org.springframework.format.annotation.NumberFormat.Style;

import br.com.mss.labqp.BasicBean;
import br.com.mss.labqp.geral.PrazoEnum;
@Entity
@Table(name = "ensaioKit", uniqueConstraints = { @UniqueConstraint(columnNames = {
		"ensaio_id", "ensaioKit_id" }) })
@SequenceGenerator(allocationSize = 1, name = "SEQUENCE", sequenceName = "SQ_ENSAIOKIT_ID")
@DisplayName("Ensaio Kit")
public class EnsaioKit extends BasicBean {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7275967666122823110L;
	/**
	 * 
	 */
	private Ensaio ensaio;
	private Ensaio ensaioKit;
	private Unidade unidade;
	private MetodoAnalitico metodoAnalitico;
	private String vmp;
	private String ld;
	private Money valor;
	private BigDecimal qtd;
	private Embalagem embalagem;
	private Conservante conservante;
	private String procAnalise;
	private Short prazoEntrega;
	private Short prazoVencimento;
	private String volume;
    private PrazoEnum prazo;
    
	@ManyToOne(fetch = FetchType.EAGER)
	@JoinColumn(name="ensaioKit_id", insertable = true, updatable = true)
	@Required
	@DisplayName("Ensaio")
	public Ensaio getEnsaioKit() {
		return ensaioKit;
	}

	public void setEnsaioKit(Ensaio ensaioKit) {
		this.ensaioKit = ensaioKit;
	}

	/**
	 * @return the ensaio
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@Required
	@JoinColumn(name="ensaio_id", insertable = true, updatable = true)
	public Ensaio getEnsaio() {
		return ensaio;
	}

	/**
	 * @param ensaio the ensaio to set
	 */
	public void setEnsaio(Ensaio ensaio) {
		this.ensaio = ensaio;
	}

	/**
	 * @return the vmp
	 */
	@Column(length = 20)
	@MaxLength(20)
	@DisplayName("VMP")
	public String getVmp() {
		return vmp;
	}

	/**
	 * @param vmp the vmp to set
	 */
	public void setVmp(String vmp) {
		this.vmp = vmp;
	}

	/**
	 * @return the unidade
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	public Unidade getUnidade() {
		return unidade;
	}

	/**
	 * @param unidade the unidade to set
	 */
	public void setUnidade(Unidade unidade) {
		this.unidade = unidade;
	}

	/**
	 * @return the metodoAnalitico
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	public MetodoAnalitico getMetodoAnalitico() {
		return metodoAnalitico;
	}

	/**
	 * @param metodoAnalitico the metodoAnalitico to set
	 */
	public void setMetodoAnalitico(MetodoAnalitico metodoAnalitico) {
		this.metodoAnalitico = metodoAnalitico;
	}

	/**
	 * @return the ld
	 */
	@Column(length = 20)
	@MaxLength(20)
	@DisplayName("LD")
	public String getLd() {
		return ld;
	}

	/**
	 * @param ld the ld to set
	 */
	public void setLd(String ld) {
		this.ld = ld;
	}

	/**
	 * @return the valor
	 */
	public Money getValor() {
		return valor;
	}

	/**
	 * @param valor the valor to set
	 */
	public void setValor(Money valor) {
		this.valor = valor;
	}

	/**
	 * @return the qtd
	 */
	@Column(precision=6,scale=2)
	@NumberFormat(style = Style.NUMBER)
	@DisplayName("Quantidade")
	public BigDecimal getQtd() {
		return qtd;
	}

	/**
	 * @param qtd the qtd to set
	 */
	public void setQtd(BigDecimal qtd) {
		this.qtd = qtd;
	}

	/**
	 * @return the embalagem
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	public Embalagem getEmbalagem() {
		return embalagem;
	}

	/**
	 * @param embalagem the embalagem to set
	 */
	public void setEmbalagem(Embalagem embalagem) {
		this.embalagem = embalagem;
	}

	/**
	 * @return the conservante
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	public Conservante getConservante() {
		return conservante;
	}

	/**
	 * @param conservante the conservante to set
	 */
	public void setConservante(Conservante conservante) {
		this.conservante = conservante;
	}

	/**
	 * @return the procAnalise
	 */
	@Column(length = 10)
	@MaxLength(10)
	@DisplayName("Procedimento de Anlise")
	public String getProcAnalise() {
		return procAnalise;
	}

	/**
	 * @param procAnalise the procAnalise to set
	 */
	public void setProcAnalise(String procAnalise) {
		this.procAnalise = procAnalise;
	}

	/**
	 * @return the prazoEntrega
	 */
	@MaxLength(3)
	@MaxValue(999)
	@DisplayName("Prazo Entrega (dias)")
	public Short getPrazoEntrega() {
		return prazoEntrega;
	}

	/**
	 * @param prazoEntrega the prazoEntrega to set
	 */
	public void setPrazoEntrega(Short prazoEntrega) {
		this.prazoEntrega = prazoEntrega;
	}

	/**
	 * @return the prazoVencimento
	 */
	@MaxLength(3)
	@MaxValue(999)
	@DisplayName("Prazo Vencimento")
	public Short getPrazoVencimento() {
		return prazoVencimento;
	}

	/**
	 * @param prazoVencimento the prazoVencimento to set
	 */
	public void setPrazoVencimento(Short prazoVencimento) {
		this.prazoVencimento = prazoVencimento;
	}

	/**
	 * @return the volume
	 */
	@Column(length = 10)
	@MaxLength(10)
	@DisplayName("Volume/Peso Mn.")
	public String getVolume() {
		return volume;
	}

	/**
	 * @param volume the volume to set
	 */
	public void setVolume(String volume) {
		this.volume = volume;
	}

	/**
	 * @return the prazo
	 */
	@DisplayName("Dia(s)/Hora(s)")
	public PrazoEnum getPrazo() {
		return prazo;
	}

	/**
	 * @param prazo the prazo to set
	 */
	public void setPrazo(PrazoEnum prazo) {
		this.prazo = prazo;
	}

}
