package br.com.mss.labqp.cadastro.entity;

import java.math.BigDecimal;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

import org.hibernate.annotations.Index;
import org.nextframework.bean.annotation.DisplayName;
import org.nextframework.types.Cnpj;
import org.nextframework.types.Cpf;
import org.nextframework.types.Telefone;
import org.nextframework.validation.annotation.MaxLength;
import org.nextframework.validation.annotation.Required;
import org.springframework.format.annotation.NumberFormat;
import org.springframework.format.annotation.NumberFormat.Style;

import br.com.mss.labqp.Pessoa;
import br.com.mss.labqp.geral.TipoPessoaEnum;
import br.com.mss.labqp.geral.entity.Endereco;

@Entity
@Table(name = "vendedor")
@DisplayName("Vendedor")
@SequenceGenerator(allocationSize = 1, name = "SEQUENCE", sequenceName = "SQ_VENDEDOR_ID")
@org.hibernate. annotations.Table (appliesTo = "vendedor", 
indexes = {@Index(name = "IDX_VENDEDOR_001",columnNames = {"cnpj"}),
		@Index(name = "IDX_VENDEDOR_002",columnNames = {"cpf"})})
public class Vendedor extends Pessoa {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3643449264922169697L;

	private TipoPessoaEnum tipoPessoa;
	private Telefone telefone;
	private Telefone celular;
	private Cnpj cnpj;
	private Cpf cpf;
	private Endereco endereco;
	private BigDecimal percentual;
	private Banco banco; 
	private String agencia;
	private String conta;
	private List<Cliente> clientes;

	@Column(length = 20)
	@MaxLength(20)
	public Telefone getTelefone() {
		return telefone;
	}

	public void setTelefone(Telefone telefone) {
		this.telefone = telefone;
	}

	@Column(length = 20)
	@MaxLength(20)
	public Telefone getCelular() {
		return celular;
	}

	public void setCelular(Telefone celular) {
		this.celular = celular;
	}

	@Column(length = 20)
	@MaxLength(20)
	public Cnpj getCnpj() {
		return cnpj;
	}

	public void setCnpj(Cnpj cnpj) {
		this.cnpj = cnpj;
	}

	@Column(length = 14)
	@MaxLength(14)
	public Cpf getCpf() {
		return cpf;
	}

	public void setCpf(Cpf cpf) {
		this.cpf = cpf;
	}

	/**
	 * @return the tipoPessoa
	 */
	@Enumerated(EnumType.ORDINAL)
	@Required
	@DisplayName("Tipo de Pessoa")
	public TipoPessoaEnum getTipoPessoa() {
		return tipoPessoa;
	}

	/**
	 * @param tipoPessoa
	 *            the tipoPessoa to set
	 */
	public void setTipoPessoa(TipoPessoaEnum tipoPessoa) {
		this.tipoPessoa = tipoPessoa;
	}


	/**
	 * @return the endereco
	 */
	@Embedded
	public Endereco getEndereco() {
		return endereco;
	}

	/**
	 * @param endereco the endereco to set
	 */
	public void setEndereco(Endereco endereco) {
		this.endereco = endereco;
	}

	/**
	 * @return the percentual
	 */
	@Column(precision=6,scale=2)
	@NumberFormat(style = Style.PERCENT)
	@DisplayName("% Comisso")
	public BigDecimal getPercentual() {
		return percentual;
	}

	/**
	 * @param percentual the percentual to set
	 */
	public void setPercentual(BigDecimal percentual) {
		this.percentual = percentual;
	}

	/**
	 * @return the banco
	 */
	public Banco getBanco() {
		return banco;
	}

	/**
	 * @param banco the banco to set
	 */
	public void setBanco(Banco banco) {
		this.banco = banco;
	}

	/**
	 * @return the agencia
	 */
	@DisplayName("Agncia")
	public String getAgencia() {
		return agencia;
	}

	/**
	 * @param agencia the agencia to set
	 */
	public void setAgencia(String agencia) {
		this.agencia = agencia;
	}

	/**
	 * @return the conta
	 */
	public String getConta() {
		return conta;
	}

	/**
	 * @param conta the conta to set
	 */
	public void setConta(String conta) {
		this.conta = conta;
	}

	@OneToMany(mappedBy = "vendedor")
	public List<Cliente> getClientes() {
		return clientes;
	}

	public void setClientes(List<Cliente> clientes) {
		this.clientes = clientes;
	}

}
