package br.com.mss.labqp.controller.modulo;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.nextframework.authorization.crud.CrudAuthorizationModule;
import org.nextframework.controller.Controller;
import org.nextframework.controller.NoActionHandlerException;
import org.nextframework.controller.crud.CrudController;
import org.nextframework.controller.crud.CrudException;
import org.nextframework.core.standard.MessageType;
import org.nextframework.core.web.WebRequestContext;
import org.nextframework.types.Money;
import org.nextframework.util.CollectionsUtil;
import org.springframework.validation.BindException;
import org.springframework.web.servlet.ModelAndView;

import br.com.mss.labqp.cadastro.entity.Cliente;
import br.com.mss.labqp.cadastro.entity.TipoDocumento;
import br.com.mss.labqp.cadastro.service.TipoDocumentoService;
import br.com.mss.labqp.financeiro.entity.Documento;
import br.com.mss.labqp.financeiro.entity.ItemDocumento;
import br.com.mss.labqp.financeiro.entity.Parcela;
import br.com.mss.labqp.financeiro.service.DocumentoService;
import br.com.mss.labqp.geral.FiltroExcluidoEnum;
import br.com.mss.labqp.geral.SituacaoDocEnum;
import br.com.mss.labqp.geral.SituacaoRequisicaoEnum;
import br.com.mss.labqp.geral.TipoDocEnum;
import br.com.mss.labqp.requisicao.entity.ItemRequisicaoAnalise;
import br.com.mss.labqp.requisicao.entity.ItemRequisicaoAnaliseFiltro;
import br.com.mss.labqp.requisicao.entity.ListaItensRequisicaoAnalise;
import br.com.mss.labqp.requisicao.entity.ListaRequisicoesAnalise;
import br.com.mss.labqp.requisicao.entity.RequisicaoAnalise;
import br.com.mss.labqp.requisicao.entity.RequisicaoAnaliseFiltro;
import br.com.mss.labqp.requisicao.service.ItemRequisicaoAnaliseService;
import br.com.mss.labqp.requisicao.service.RequisicaoAnaliseService;

@Controller(path = "/requisicao/crud/RequisicaoAnalise", authorizationModule = CrudAuthorizationModule.class)
public class RequisicaoAnaliseCrud extends
		CrudController<RequisicaoAnaliseFiltro, RequisicaoAnalise, RequisicaoAnalise> {
	
	private static final String PARAM_GERAR_LAUDO = "GERAR_LAUDO";
	TipoDocumentoService tipoDocumentoService;
	DocumentoService documentoService;
    RequisicaoAnaliseService requisicaoAnaliseService;
    ItemRequisicaoAnaliseService itemRequisicaoAnaliseService;
    
	/* (non-Javadoc)
	 * @see org.nextframework.controller.crud.CrudController#validate(java.lang.Object, org.springframework.validation.BindException, java.lang.String)
	 */
	@Override
	protected void validate(Object obj, BindException errors, String acao) {
		if ("salvar".equals(acao)) {
			RequisicaoAnalise ra = (RequisicaoAnalise) obj;
			if (ra.getSituacaoRequisicaoEnum() == SituacaoRequisicaoEnum.LAUDO_GERADO) {
				errors.reject("Validao",
						"J foi gerado Laudo para essa Requisio. No deve ser alterada!");
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.nextframework.controller.crud.CrudController#excluir(org.nextframework.core.web.WebRequestContext, java.lang.Object)
	 */
	@Override
	protected void excluir(WebRequestContext request, RequisicaoAnalise bean)
			throws Exception {
		bean = genericService.loadForEntrada(bean);
		if (bean.getSituacaoRequisicaoEnum() == SituacaoRequisicaoEnum.LAUDO_GERADO) {
			throw new Exception ("J foi gerado Laudo para essa Requisio. No deve ser excluda!" );
		}else {
			super.excluir(request, bean);
		}
	}
	
	public ModelAndView gerarLaudo(WebRequestContext request, RequisicaoAnalise bean) throws Exception {
		setUserAttribute(PARAM_GERAR_LAUDO, "N");
		if (bean.getSituacaoRequisicaoEnum() == SituacaoRequisicaoEnum.RESULTADOS_INCLUIDOS) {
			bean.setSituacaoRequisicaoEnum(SituacaoRequisicaoEnum.LAUDO_GERADO);
			this.salvar(request, bean);
			
			setUserAttribute(PARAM_GERAR_LAUDO, "S");
			request.addMessage("Laudo gerado com Sucesso!", MessageType.INFO);
			return new ModelAndView("redirect:/requisicao/relatorio/LaudoRequisicao?id="+bean.getId());
		}else {
			request.addError("Ocorreu um erro e o Laudo no foi gerado. Verifique!");
		}
		bean = genericService.loadForEntrada(bean);
	 	return this.continueOnAction(EDITAR,bean);
	}
	
	public String prepararReqFinanceiro(WebRequestContext request) {
		request.setAttribute("listaRequisicoesAnalise", new ListaRequisicoesAnalise());
		request.setAttribute("requisicaoAnalise", new RequisicaoAnalise());
		return "jspGerarFinaceiro";
	}
	
	public String prepararAnalisesRealizadas(WebRequestContext request) {
		return "jspGerarFir";
	}

	public String gerarFinaceiro(WebRequestContext request, ListaRequisicoesAnalise listaRequisicoesAnalise ) throws Exception {
		try {
			List<RequisicaoAnalise> raList = null;
			if (listaRequisicoesAnalise != null) {
				raList = listaRequisicoesAnalise.getRequisicoesAnalise();
			}
			if (raList != null && raList.size() > 0) {
				List<RequisicaoAnalise> raSalvarList 	= new ArrayList<RequisicaoAnalise>();
				Cliente cliente = null;
				for (RequisicaoAnalise requisicaoAnalise : raList) {
					if (requisicaoAnalise.getSelecionado() == true) {
						
						requisicaoAnalise = genericService.loadForEntrada(requisicaoAnalise);
							
						if (cliente == null)
							cliente = requisicaoAnalise.getCliente();
						
						requisicaoAnalise.setSituacaoRequisicaoEnum(SituacaoRequisicaoEnum.FINANCEIRO_GERADO);
						raSalvarList.add(requisicaoAnalise);
					}
				}
				geraDocFinanceiro(request, raSalvarList,cliente);
			}
		} catch (Exception e) {
			request.addError("Ocorreu um erro na incluso do documento financeiro! /n" + e.getMessage() );
		}
		
		request.setAttribute("listaRequisicoesAnalise", new ListaRequisicoesAnalise());
		request.setAttribute("requisicaoAnalise", new RequisicaoAnalise());
		request.addMessage("Gerao do financeiro para as requisies selecionadas executada com sucesso!");
		return "jspGerarFinaceiro";
	}

	public String pesquisarReqComLaudo(WebRequestContext request,RequisicaoAnalise bean) {
		ListaRequisicoesAnalise listaRequisicoesAnalise = new ListaRequisicoesAnalise();
		RequisicaoAnaliseFiltro raFiltro = new RequisicaoAnaliseFiltro();
		raFiltro.setCliente(bean.getCliente());
		raFiltro.setSituacaoRequisicaoEnum(SituacaoRequisicaoEnum.LAUDO_GERADO);
		listaRequisicoesAnalise.setRequisicoesAnalise(requisicaoAnaliseService.findByFiltro(raFiltro));
		if (listaRequisicoesAnalise.getRequisicoesAnalise().size() == 0) {
			request.addError("No h dados para essa seleo!");
		}
		request.setAttribute("requisicaoAnalise", bean);
		request.setAttribute("listaRequisicoesAnalise", listaRequisicoesAnalise);
		return "jspGerarFinaceiro";
	}

	/**
	 * @param requisicaoAnaliseService the requisicaoAnaliseService to set
	 */
	public void setRequisicaoAnaliseService(
			RequisicaoAnaliseService requisicaoAnaliseService) {
		this.requisicaoAnaliseService = requisicaoAnaliseService;
	}
	
	private void geraDocFinanceiro(WebRequestContext request, List<RequisicaoAnalise> raSalvarList, Cliente cliente ) throws Exception{
		try {
			if (raSalvarList != null && raSalvarList.size() > 0) {
				Documento doc = new Documento();
				doc.setCliente(cliente);
				doc.setDataInc(new Date());
				doc.setDataProcessamento(new Date());
				doc.setFiltroExcluidoEnum(FiltroExcluidoEnum.NAO);
				doc.setSituacaoDocEnum(SituacaoDocEnum.ABERTO);
				doc.setTipoDocEnum(TipoDocEnum.ENTRADA);
				doc.setUserInc( getUser().getLogin());
				TipoDocumento td = new TipoDocumento();
				
				/*
				 * O cdigo um (1) tem que representar o tipo de documento "requisio de anlises"
				 */
				td.setId(new Long("1")); 
				td = tipoDocumentoService.load(td);
				List<ItemDocumento> idList = new ArrayList<ItemDocumento>();
				Money vlrTotalDoc = new Money("0");
				for (RequisicaoAnalise requisicaoAnalise : raSalvarList) {
					ItemDocumento id = new ItemDocumento();
					id.setIdDoc(requisicaoAnalise.getId());
					id.setDescricao("Documento gerado apartir da RA " + requisicaoAnalise.getId());
					id.setTipoDocumento(td);
					
					Money vlrTotalReq = new Money("0");
					for (ItemRequisicaoAnalise  ira : requisicaoAnalise.getItensRequisicaoAnalise()) {
						vlrTotalReq = vlrTotalReq.add(ira.getValor());
					}
					
					id.setValor(vlrTotalReq);
					idList.add(id);
					vlrTotalDoc = vlrTotalDoc.add(vlrTotalReq);
					this.salvar(request, requisicaoAnalise);
				}
				doc.setItensDocumento(idList);
				
				Parcela parcela = new Parcela();
				
				parcela.setDataInc(new Date());
				parcela.setUserInc(getUser().getLogin());
				parcela.setDataProcessamento(new Date());
				parcela.setDataVencimento( new Date());
				parcela.setParcelaSeq(new Short("1"));
				parcela.setFiltroExcluidoEnum(FiltroExcluidoEnum.NAO);
				parcela.setValorParcela(vlrTotalDoc);
				List<Parcela> parcelaList = new ArrayList<Parcela>();
				parcelaList.add(parcela);
				doc.setParcelas(parcelaList);
				documentoService.saveOrUpdate(doc);
			}
			
		} catch (Exception e) {
	//		throw new Exception("Ocorreu um erro na incluso do documento financeiro!",  e);
			request.addError("Ocorreu um erro na incluso do documento financeiro! /n" + e.getMessage() );
		}
	}	

	/**
	 * @param tipoDocumentoService the tipoDocumentoService to set
	 */
	public void setTipoDocumentoService(TipoDocumentoService tipoDocumentoService) {
		this.tipoDocumentoService = tipoDocumentoService;
	}

	/**
	 * @param documentoService the documentoService to set
	 */
	public void setDocumentoService(DocumentoService documentoService) {
		this.documentoService = documentoService;
	}

	/* (non-Javadoc)
	 * @see org.nextframework.controller.crud.CrudController#entrada(org.nextframework.core.web.WebRequestContext, java.lang.Object)
	 * Verifica se a situao da Requisio de Anlise  de "LAUDO_GERADO" ou "FINANCEIRO_GERADO". Se for gera um exceo. Somente quando for Editar
	 */
	@Override
	protected void entrada(WebRequestContext request, RequisicaoAnalise form)
			throws Exception {
		if (request.getParameter(ACTION_PARAMETER).equals(EDITAR) &&
		(form.getSituacaoRequisicaoEnum() == SituacaoRequisicaoEnum.LAUDO_GERADO ||
				form.getSituacaoRequisicaoEnum() == SituacaoRequisicaoEnum.FINANCEIRO_GERADO)) {
			throw new CrudException(ACTION_PARAMETER, new Exception( "J foi gerado Laudo para essa Requisio. No deve ser alterada!"));
		}
		
		super.entrada(request, form);
	}
	
	public String inclusaoResultados(WebRequestContext request, ListaItensRequisicaoAnalise form) throws Exception {
		try {
			List<ItemRequisicaoAnalise> itemRaList = form.getItensRequisicaoAnalise();
			if (itemRaList != null && itemRaList.size() > 0) {
				RequisicaoAnalise ra;
				List<RequisicaoAnalise> raArray = new ArrayList<RequisicaoAnalise>();
				for (ItemRequisicaoAnalise itemRa : itemRaList) {
					ItemRequisicaoAnalise itemRaSalvar = itemRequisicaoAnaliseService.loadForEntrada(itemRa);
					ra = itemRaSalvar.getRequisicaoAnalise();
					itemRaSalvar.setResultado(itemRa.getResultado());
					itemRaSalvar.setMetodoAnalitico(itemRa.getMetodoAnalitico());
					itemRaSalvar.setUnidade(itemRa.getUnidade());
					itemRaSalvar.setVmp(itemRa.getVmp());
					itemRequisicaoAnaliseService.saveOrUpdate(itemRaSalvar);
					if (!CollectionsUtil.contains(raArray, ra)) {
						raArray.add(ra);
					}
				}
				for (RequisicaoAnalise raObj : raArray) {
					if (raObj != null) {
						ra = requisicaoAnaliseService.loadForEntrada(raObj);
						requisicaoAnaliseService.saveOrUpdate(ra);
					}
				}
				form.setItensRequisicaoAnalise(new ArrayList<ItemRequisicaoAnalise>());
				request.addMessage("Resultados includos com sucesso!");
			}
		} catch (Exception e) {
			request.addError("Ocorreu um erro na incluso dos resultados! /n" + e.getMessage() );
		}
		
		request.setAttribute("itemRequisicaoAnaliseFiltro", new ItemRequisicaoAnaliseFiltro());
		request.setAttribute("listaItensRequisicaoAnalise",form);
		return "crud/jspInclusaoResultados";
	}

	public String pesquisarItensSemResultado(WebRequestContext request,ItemRequisicaoAnaliseFiltro filtro) throws Exception {
		ListaItensRequisicaoAnalise listaItensRequisicaoAnalise = new ListaItensRequisicaoAnalise();
		try {
			String arg = getParameter("arg");
			if (arg == null ) {
				listaItensRequisicaoAnalise.setItensRequisicaoAnalise(itemRequisicaoAnaliseService.findByFiltro(filtro));
				if (listaItensRequisicaoAnalise.getItensRequisicaoAnalise().size() == 0) {
					request.addError("No h dados para essa seleo!");
				}
			} else {
				filtro = new ItemRequisicaoAnaliseFiltro();
				filtro.setSituacaoRequisicaoEnum(SituacaoRequisicaoEnum.AGUARDANDO_RESULTADOS);
				listaItensRequisicaoAnalise.setItensRequisicaoAnalise(new ArrayList<ItemRequisicaoAnalise>());
			}
		} catch (Exception e) {
			request.addError("Ocorreu um erro na pesquisa. Favor verifique! /n" + e.getMessage() );
		}
		request.setAttribute("itemRequisicaoAnaliseFiltro", filtro);
		request.setAttribute("listaItensRequisicaoAnalise", listaItensRequisicaoAnalise);
		return "crud/jspInclusaoResultados";
	}

	
	/**
	 * @param itemRequisicaoAnaliseService the itemRequisicaoAnaliseService to set
	 */
	public void setItemRequisicaoAnaliseService(
			ItemRequisicaoAnaliseService itemRequisicaoAnaliseService) {
		this.itemRequisicaoAnaliseService = itemRequisicaoAnaliseService;
	}

	@Override
	protected ModelAndView handleRequestInternal(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
	      try{
		         return super.handleRequestInternal(request, response);
		      }catch(NoActionHandlerException exception){ //se nao achar a action
		         return sendRedirectToAction(null); //volta para a action default
		      }
	}
}	