package br.com.mss.labqp.financeiro.dao;

import java.util.Date;
import java.util.List;

import org.nextframework.controller.crud.FiltroListagem;
import org.nextframework.persistence.QueryBuilder;
import org.nextframework.persistence.SaveOrUpdateStrategy;

import br.com.mss.labqp.GenericAuditDAO;
import br.com.mss.labqp.financeiro.entity.Documento;
import br.com.mss.labqp.financeiro.entity.DocumentoFiltro;
import br.com.mss.labqp.geral.FiltroExcluidoEnum;
import br.com.mss.labqp.geral.SituacaoDocEnum;
import br.com.mss.labqp.requisicao.entity.RequisicaoAnaliseFiltro;

public class DocumentoDAO extends GenericAuditDAO<Documento> {

	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateListagemQuery(org.nextframework.persistence.QueryBuilder, org.nextframework.controller.crud.FiltroListagem)
	 */
	@Override
	public void updateListagemQuery(QueryBuilder<Documento> query,
			FiltroListagem _filtro) {
		DocumentoFiltro filtro = (DocumentoFiltro) _filtro;
		query.joinFetch("documento.cliente");
		query.where("documento.id = ? ",filtro.getId() );
		query.where("documento.cliente = ?", filtro.getCliente());
		query.where("documento.tipoDocEnum = ?", filtro.getTipoDocEnum());
		query.where("documento.situacaoDocEnum = ?", filtro.getSituacaoDocEnum());
		query.where("documento.numeroNf = ?", filtro.getNumeroNf());
		if (filtro.getFiltroExcluido() != null) {
			if (filtro.getFiltroExcluido() == FiltroExcluidoEnum.SIM)
				query.where("documento.dataExc is not null");
			else
				query.where("documento.dataExc is null");

		}
		query.orderBy("documento.cliente, dataEmissao");
	}

	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateSaveOrUpdate(org.nextframework.persistence.SaveOrUpdateStrategy)
	 */
	@Override
	public void updateSaveOrUpdate(SaveOrUpdateStrategy save) {
		save.saveOrUpdateManaged("itensDocumento");
		save.saveOrUpdateManaged("parcelas");
		save.saveOrUpdateManaged("impostosDocumento");
		super.updateSaveOrUpdate(save);
	}
	
	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateEntradaQuery(org.nextframework.persistence.QueryBuilder)
	 */
	@Override
	public void updateEntradaQuery(QueryBuilder<Documento> query) {
		query.fetchCollection("itensDocumento");
		query.fetchCollection("parcelas");
		query.fetchCollection("impostosDocumento");
		query.joinFetch("documento.cliente");
		super.updateEntradaQuery(query);
	}
	
	public void atualizaVencimento(RequisicaoAnaliseFiltro filtro) {
		Date[] datas = { filtro.getDataI(), filtro.getDataF() };
		QueryBuilder<Documento> queryAtuVenc = this.query();
		queryAtuVenc.where("documento.ano = ?",filtro.getAno() );
		queryAtuVenc.where("documento.mes = ?",filtro.getMes());
		queryAtuVenc.where("documento.dataProcessamento between ? and ?",datas);
		queryAtuVenc.where("documento.cliente = ?",filtro.getCliente());
		queryAtuVenc.where("documento.situacaoDocEnum = ?", SituacaoDocEnum.ABERTO);

		List<Documento> docs = queryAtuVenc.list();
		for (Documento documento : docs) {
			documento.setDataEmissao(new Date());
			documento.setDataVencimento(filtro.getVencimento());
			documento.setNumeroNf(filtro.getNf());
			this.getHibernateTemplate().saveOrUpdate(documento);
		}
	}
	
}
