package br.com.mss.labqp.financeiro.dao;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.hibernate.Query;
import org.nextframework.controller.crud.FiltroListagem;
import org.nextframework.persistence.QueryBuilder;
import org.nextframework.types.Money;

import br.com.mss.labqp.GenericAuditDAO;
import br.com.mss.labqp.cadastro.dao.ClienteDAO;
import br.com.mss.labqp.cadastro.entity.Cliente;
import br.com.mss.labqp.financeiro.entity.Parcela;
import br.com.mss.labqp.financeiro.entity.ParcelaFiltro;
import br.com.mss.labqp.financeiro.entity.TotalClientePeriodo;
import br.com.mss.labqp.geral.FiltroExcluidoEnum;
import br.com.mss.labqp.geral.SituacaoDocEnum;

public class ParcelaDAO extends GenericAuditDAO<Parcela> {

	private ClienteDAO clienteDAO;
	
	/**
	 * @param clienteDAO the clienteDAO to set
	 */
	public void setClienteDAO(ClienteDAO clienteDAO) {
		this.clienteDAO = clienteDAO;
	}

	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateEntradaQuery(org.nextframework.persistence.QueryBuilder)
	 */
	@Override
	public void updateEntradaQuery(QueryBuilder<Parcela> query) {
		query.joinFetch("parcela.documento");
		query.joinFetch("parcela.documento.cliente");
		super.updateEntradaQuery(query);
	}

	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateListagemQuery(org.nextframework.persistence.QueryBuilder, org.nextframework.controller.crud.FiltroListagem)
	 */
	@Override
	public void updateListagemQuery(QueryBuilder<Parcela> query,
			FiltroListagem _filtro) {
		ParcelaFiltro filtro = (ParcelaFiltro) _filtro;
		query = montaQuery(filtro, query);
		query.orderBy("parcela.dataVencimento, parcela.documento.cliente.nome");
	}

	private QueryBuilder<Parcela> montaQuery(ParcelaFiltro filtro,
			QueryBuilder<Parcela> query) {
		query.joinFetch("parcela.documento");
		query.joinFetch("parcela.documento.cliente");
		Date[] datasProc = { filtro.getDataProcIni(), filtro.getDataProcFim() };
		Date[] datasVenc = { filtro.getDataVencIni(), filtro.getDataVencFim() };
		Date[] datasPgto = { filtro.getDataPagIni(), filtro.getDataPagFim() };

		query.where("parcela.documento = ? ", filtro.getDocumento())
				.where("parcela.documento.cliente = ? ",
						filtro.getCliente())
				.where("parcela.dataPagamento between ? and ? ", datasPgto)
				.where("parcela.dataProcessamento between ? and ? ", datasProc)
				.where("parcela.dataVencimento between ? and ? ", datasVenc)
				.where("parcela.documento.tipoDocEnum = ? ",
						filtro.getTipoDocEnum());				
		if (filtro.getSituacaoDocEnum() != null) {
			if (filtro.getSituacaoDocEnum() == SituacaoDocEnum.ABERTO)
				query.where("parcela.dataPagamento is null");
			else
				query.where("parcela.dataPagamento is not null");

		}
		if (filtro.getFiltroExcluido() != null) {
			if (filtro.getFiltroExcluido() == FiltroExcluidoEnum.SIM)
				query.where("parcela.dataExc is not null");
			else
				query.where("parcela.dataExc is null");

		}
		return query;
	}
	
	public List<TotalClientePeriodo> totalClientePeriodo(ParcelaFiltro filtro) {
		StringBuilder queryString = new StringBuilder();
		queryString.append("select p.documento.cliente.id,  sum(p.valorParcela)");
		queryString.append(" from Parcela as p ");
		queryString.append(" where p.dataVencimento between coalesce(:dataVencIni,p.dataVencimento) and coalesce(:dataVencFim,p.dataVencimento)");
		queryString.append(" and p.documento.cliente = coalesce(:cliente,p.documento.cliente)");
		queryString.append(" and p.documento.tipoDocEnum = coalesce(:tipoDocEnum,p.documento.tipoDocEnum)");
		queryString.append(" and p.documento.situacaoDocEnum = coalesce(:situacaoDocEnum,p.documento.situacaoDocEnum)");
		queryString.append(" and p.dataExc is null");
		queryString.append(" group by p.documento.cliente.id ");
		
	
		Query select = getSession().createQuery(queryString.toString());
		select.setParameter("dataVencIni", filtro.getDataVencIni());
		select.setParameter("dataVencFim", filtro.getDataVencFim());
		select.setParameter("cliente", filtro.getCliente());
		select.setParameter("tipoDocEnum", filtro.getTipoDocEnum());
		select.setParameter("situacaoDocEnum", filtro.getSituacaoDocEnum());
	
		List<TotalClientePeriodo> totalClienteList = new ArrayList<TotalClientePeriodo>();
		@SuppressWarnings("unchecked")
		List<Object[]> lista = select.list();
		TotalClientePeriodo totalClientePeriodo;
		Cliente cliente;
		for (Object[] object : lista) {
			
			cliente = new Cliente();
			cliente.setId((Long) object[0]);
			cliente = clienteDAO.load(cliente);
			totalClientePeriodo = new TotalClientePeriodo();
			totalClientePeriodo.setCliente(cliente);
			totalClientePeriodo.setTotal(new Money((Long) object[1],true));
			totalClienteList.add(totalClientePeriodo);			
		}
	
		return totalClienteList;		
	}
 	
}
