package br.com.mss.labqp.financeiro.entity;

import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Formula;
import org.nextframework.bean.annotation.DescriptionProperty;
import org.nextframework.bean.annotation.DisplayName;
import org.nextframework.types.Money;
import org.nextframework.validation.annotation.MaxLength;
import org.nextframework.validation.annotation.MaxValue;
import org.nextframework.validation.annotation.Required;
import org.springframework.format.annotation.DateTimeFormat;

import br.com.mss.labqp.BasicBeanAudit;
import br.com.mss.labqp.cadastro.entity.Cliente;
import br.com.mss.labqp.geral.MesEnum;
import br.com.mss.labqp.geral.SituacaoDocEnum;
import br.com.mss.labqp.geral.TipoDocEnum;


@Entity
@Table(name = "documento")
@DisplayName("Documento")
@SequenceGenerator(allocationSize = 1, name = "SEQUENCE", sequenceName = "SQ_DOCUMENTO_ID")
public class Documento extends BasicBeanAudit{

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 7954995385069395214L;
	private Date dataProcessamento;
	private Date dataEmissao;
	private Date dataVencimento;
	private MesEnum mes;
	private Short ano;
	private Cliente cliente;
	private String observacao;
	private String numeroNf;
	private TipoDocEnum tipoDocEnum;
	private SituacaoDocEnum situacaoDocEnum;
	private Money valorTotal;
	private Long documentoId; // Formula - Usado como DescriptionProperty 
	private Money valorImpostos;
	private Money valorLiquido;
	
	private List<Parcela> parcelas;
	private List<ItemDocumento> itensDocumento;
	private List<ImpostoDocumento> impostosDocumento;
	
	
	/**
	 * @return the datahoraColeta
	 */
	
	@DateTimeFormat()
	@Temporal(TemporalType.DATE)
	public Date getDataEmissao() {
		return dataEmissao;
	}
	/**
	 * @param datahoraColeta the datahoraColeta to set
	 */
	public void setDataEmissao(Date dataEmissao) {
		this.dataEmissao = dataEmissao;
	}
	/**
	 * @return the cliente
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@Required
	public Cliente getCliente() {
		return cliente;
	}
	/**
	 * @param cliente the cliente to set
	 */
	public void setCliente(Cliente cliente) {
		this.cliente = cliente;
	}
	/**
	 * @return the observacao
	 */
	@Column(length = 150)
	@MaxLength(150)
	@DisplayName("Observao")
	public String getObservacao() {
		return observacao;
	}
	/**
	 * @param observacao the observacao to set
	 */
	public void setObservacao(String observacao) {
		this.observacao = observacao;
	}
	/**
	 * @return the numeroNf
	 */
	@DisplayName("Nmero NF")
	public String getNumeroNf() {
		return numeroNf;
	}
	/**
	 * @param numeroNf the numeroNf to set
	 */
	public void setNumeroNf(String numeroNf) {
		this.numeroNf = numeroNf;
	}
	/**
	 * @return the situacaoDocEnum
	 */
	@DisplayName("Situaao Documento")
	@Required
	public SituacaoDocEnum getSituacaoDocEnum() {
		return situacaoDocEnum;
	}
	/**
	 * @param situacaoDocEnum the situacaoDocEnum to set
	 */
	public void setSituacaoDocEnum(SituacaoDocEnum situacaoDocEnum) {
		this.situacaoDocEnum = situacaoDocEnum;
	}
	/**
	 * @return the tipoDocEnum
	 */
	@DisplayName("Tipo Faturamento")
	@Required
	public TipoDocEnum getTipoDocEnum() {
		return tipoDocEnum;
	}
	/**
	 * @param tipoDocEnum the tipoDocEnum to set
	 */
	public void setTipoDocEnum(TipoDocEnum tipoDocEnum) {
		this.tipoDocEnum = tipoDocEnum;
	}
	/**
	 * @return the itensDocumento
	 */
	@OneToMany(mappedBy = "documento")
	@DisplayName("Itens do Documento")
	public List<ItemDocumento> getItensDocumento() {
		return itensDocumento;
	}
	/**
	 * @param itensDocumento the itensDocumento to set
	 */
	public void setItensDocumento(List<ItemDocumento> itensDocumento) {
		this.itensDocumento = itensDocumento;
	}
	/**
	 * @return the valorTotal
	 */
	//@Formula("(select min(o.creation_date) from Orders o where o.customer_id = id)")
	@Formula("(select sum(i.valor) from itemDocumento i where i.documento_id = id) ")
	public Money getValorTotal() {
		return valorTotal;
	}
	/**
	 * @param valorTotal the valorTotal to set
	 */
	public void setValorTotal(Money valorTotal) {
		this.valorTotal = valorTotal;
	}
	/**
	 * @return the dataProcessamento
	 */
	@DateTimeFormat()
	@Temporal(TemporalType.DATE)
	@Required
	public Date getDataProcessamento() {
		return dataProcessamento;
	}
	/**
	 * @param dataProcessamento the dataProcessamento to set
	 */
	public void setDataProcessamento(Date dataProcessamento) {
		this.dataProcessamento = dataProcessamento;
	}
	/**
	 * @return the parcelas
	 */
	@OneToMany(mappedBy = "documento")
	public List<Parcela> getParcelas() {
		return parcelas;
	}
	/**
	 * @param parcelas the parcelas to set
	 */
	public void setParcelas(List<Parcela> parcelas) {
		this.parcelas = parcelas;
	}
	/**
	 * @return the documentoId
	 */
	@Formula("id")
	@DisplayName("Id")
	@DescriptionProperty
	public Long getDocumentoId() {
		return documentoId;
	}
	/**
	 * @param documentoId the documentoId to set
	 */
	public void setDocumentoId(Long documentoId) {
		this.documentoId = documentoId;
	}
	/**
	 * @return the impostosDocumento
	 */
	@OneToMany(mappedBy = "documento")
	@DisplayName("Impostos")
	public List<ImpostoDocumento> getImpostosDocumento() {
		return impostosDocumento;
	}
	/**
	 * @param impostosDocumento the impostosDocumento to set
	 */
	public void setImpostosDocumento(List<ImpostoDocumento> impostosDocumento) {
		this.impostosDocumento = impostosDocumento;
	}
	/**
	 * @return the valorLiquido
	 */
	@Transient
	public Money getValorLiquido() {
		return valorLiquido;
	}
	/**
	 * @param valorLiquido the valorLiquido to set
	 */
	public void setValorLiquido(Money valorLiquido) {
		this.valorLiquido = valorLiquido;
	}
	/**
	 * @return the dataVencimento
	 */
	@DateTimeFormat()
	@Temporal(TemporalType.DATE)
	public Date getDataVencimento() {
		return dataVencimento;
	}
	/**
	 * @param dataVencimento the dataVencimento to set
	 */
	public void setDataVencimento(Date dataVencimento) {
		this.dataVencimento = dataVencimento;
	}
	/**
	 * @return the mes
	 */
	@DisplayName("Ms")
	public MesEnum getMes() {
		return mes;
	}
	/**
	 * @param mes the mes to set
	 */
	public void setMes(MesEnum mes) {
		this.mes = mes;
	}
	/**
	 * @return the ano
	 */
	@MaxLength(4)
	@MaxValue(3000)
	public Short getAno() {
		return ano;
	}
	/**
	 * @param ano the ano to set
	 */
	public void setAno(Short ano) {
		this.ano = ano;
	}
	/**
	 * @return the valorImpostos
	 */
	@Transient
	public Money getValorImpostos() {
		return valorImpostos;
	}
	/**
	 * @param valorImpostos the valorImpostos to set
	 */
	public void setValorImpostos(Money valorImpostos) {
		this.valorImpostos = valorImpostos;
	}
}
