package br.com.mss.labqp.financeiro.entity;

import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.annotations.Formula;
import org.nextframework.bean.annotation.DescriptionProperty;
import org.nextframework.bean.annotation.DisplayName;
import org.nextframework.types.Money;
import org.nextframework.validation.annotation.MaxLength;
import org.nextframework.validation.annotation.Required;
import org.springframework.format.annotation.DateTimeFormat;

import br.com.mss.labqp.BasicBeanAudit;


@Entity
@Table(name = "parcela")
@DisplayName("Parcela")
@SequenceGenerator(allocationSize = 1, name = "SEQUENCE", sequenceName = "SQ_PARCELA_ID")
public class Parcela extends BasicBeanAudit{

	/**
	 * 
	 */
	private static final long serialVersionUID = 7954995385069395214L;
	private Documento documento;
	private Short parcelaSeq;
	private Date dataProcessamento;
	private Date dataVencimento;
	private Date dataPagamento;
	private Money valorParcela;
	private Money valorPagamento;
	private Long codigoId; // Formula - Usado como DescriptionProperty
	/**
	 * @return the documento
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@Required
	public Documento getDocumento() {
		return documento;
	}
	/**
	 * @param documento the documento to set
	 */
	public void setDocumento(Documento documento) {
		this.documento = documento;
	}
	
	/**
	 * @return the parcelaSeq
	 */
	@Required
	@MaxLength(4)
	public Short getParcelaSeq() {
		return parcelaSeq;
	}

	/**
	 * @param parcelaSeq
	 *            the parcelaSeq to set
	 */
	public void setParcelaSeq(Short parcelaSeq) {
		this.parcelaSeq = parcelaSeq;
	}

	/**
	 * @return the dataProcessamento
	 */
	@DateTimeFormat
	@Temporal(TemporalType.DATE)
	@Required
	@DisplayName("Data Proc.")
	public Date getDataProcessamento() {
		return dataProcessamento;
	}

	/**
	 * @param dataProcessamento
	 *            the dataProcessamento to set
	 */
	public void setDataProcessamento(Date dataProcessamento) {
		this.dataProcessamento = dataProcessamento;
	}

	/**
	 * @return the dataVencimento
	 */
	@DateTimeFormat
	@Temporal(TemporalType.DATE)
	@Required
	@DisplayName("Data Venc.")
	public Date getDataVencimento() {
		return dataVencimento;
	}

	/**
	 * @param dataVencimento
	 *            the dataVencimento to set
	 */
	public void setDataVencimento(Date dataVencimento) {
		this.dataVencimento = dataVencimento;
	}

	/**
	 * @return the dataPagamento
	 */
	@DateTimeFormat
	@Temporal(TemporalType.DATE)
	@DisplayName("Data Pgto.")
	public Date getDataPagamento() {
		return dataPagamento;
	}

	/**
	 * @param dataPagamento
	 *            the dataPagamento to set
	 */
	public void setDataPagamento(Date dataPagamento) {
		this.dataPagamento = dataPagamento;
	}

	/**
	 * @return the valorParcela
	 */
	@DisplayName("Valor da Parcela")
	public Money getValorParcela() {
		return valorParcela;
	}

	/**
	 * @param valorParcela
	 *            the valorParcela to set
	 */
	public void setValorParcela(Money valorParcela) {
		this.valorParcela = valorParcela;
	}

	/**
	 * @return the valorPagamento
	 */
	@DisplayName("Valor de Pagamento")
	public Money getValorPagamento() {
		return valorPagamento;
	}

	/**
	 * @param valorPagamento
	 *            the valorPagamento to set
	 */
	public void setValorPagamento(Money valorPagamento) {
		this.valorPagamento = valorPagamento;
	}
	/**
	 * @return the codigoId
	 */
	@Formula("id")
	@DisplayName("Id")
	@DescriptionProperty
	public Long getCodigoId() {
		return codigoId;
	}
	/**
	 * @param codigoId the codigoId to set
	 */
	public void setCodigoId(Long codigoId) {
		this.codigoId = codigoId;
	}
}
