package br.com.mss.labqp.financeiro.service;

import java.util.List;

import org.nextframework.service.GenericService;
import org.nextframework.types.Money;

import br.com.mss.labqp.financeiro.dao.DocumentoDAO;
import br.com.mss.labqp.financeiro.entity.Documento;
import br.com.mss.labqp.financeiro.entity.ImpostoDocumento;
import br.com.mss.labqp.financeiro.entity.Parcela;
import br.com.mss.labqp.geral.SituacaoDocEnum;
import br.com.mss.labqp.requisicao.entity.RequisicaoAnaliseFiltro;

public class DocumentoService extends GenericService<Documento> {

	DocumentoDAO documentoDAO;
	
	/**
	 * @param documentoDAO the documentoDAO to set
	 */
	public void setDocumentoDAO(DocumentoDAO documentoDAO) {
		this.documentoDAO = documentoDAO;
	}

	/* (non-Javadoc)
	 * @see org.nextframework.service.GenericService#saveOrUpdate(java.lang.Object)
	 */
	@Override
	public void saveOrUpdate(Documento bean) {
		// verifica se as parcelas foram todas pagas para o documento e atualiza a situao
		if (verificaParcelasDocumento(bean)) {
			bean.setSituacaoDocEnum(SituacaoDocEnum.FECHADO);
		} else {
			bean.setSituacaoDocEnum(SituacaoDocEnum.ABERTO);
		}
		super.saveOrUpdate(bean);
	}

	public Boolean verificaParcelasDocumento(Documento bean) {
		List<Parcela> parcelas = bean.getParcelas();
		Money totalParcela = new Money("0");
		Money totalPgto = new Money("0");
		if (parcelas != null && parcelas.size()>0) {
			for (Parcela parcela : parcelas) {
				if (parcela.getValorParcela() != null) 
					totalParcela = totalParcela.add(parcela.getValorParcela());
				if (parcela.getValorPagamento()  != null)
					totalPgto = totalPgto.add(parcela.getValorPagamento());
			}
		}
		return totalPgto.compareTo(totalParcela) <= 0;
	}

	/* (non-Javadoc)
	 * @see org.nextframework.service.GenericService#loadForEntrada(java.lang.Object)
	 */
	@Override
	public Documento loadForEntrada(Documento bean) {
		bean = super.loadForEntrada(bean); 
		List<ImpostoDocumento> idList = bean.getImpostosDocumento();
		bean.setValorImpostos(new Money("0"));
		bean.setValorLiquido(new Money("0"));
		if (idList != null && idList.size() > 0) {
			Money valorLiquido = new Money("0");
			for (ImpostoDocumento impostoDocumento : idList) {
				valorLiquido = valorLiquido.add(impostoDocumento.getValor());
			}
			bean.setValorImpostos(valorLiquido);
			bean.setValorLiquido(bean.getValorTotal().subtract(valorLiquido));
		}
		return bean;
	}

	public void atualizaVencimento(RequisicaoAnaliseFiltro filtro) {
		documentoDAO.atualizaVencimento(filtro);
	}

}
