package br.com.mss.labqp.orcamento.entity;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Formula;
import org.nextframework.bean.annotation.DescriptionProperty;
import org.nextframework.bean.annotation.DisplayName;
import org.nextframework.types.Money;
import org.nextframework.validation.annotation.MaxLength;
import org.nextframework.validation.annotation.Required;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.format.annotation.NumberFormat;
import org.springframework.format.annotation.NumberFormat.Style;

import br.com.mss.labqp.BasicBeanAudit;
import br.com.mss.labqp.cadastro.entity.Cliente;

@Entity
@Table(name = "orcamento")
@DisplayName("Oramento")
@SequenceGenerator(allocationSize = 1, name = "SEQUENCE", sequenceName = "SQ_ORCAMENTO_ID")
public class Orcamento extends BasicBeanAudit{

	/**
	 * 
	 */
	private static final long serialVersionUID = -4132200998459548197L;
	
	private Date data;
	private Short validade;
	private BigDecimal desconto;
	private Cliente cliente;
	private Money valorTotal; 
	private Money ValorDesconto;
	private Money valorTotalLiq;
	private Long codigoId; // Formula - usado como DescriptionProperty 

	private Set<OrcamentoTipoAmostra> tiposAmostras;
	private Set<ItensAdicionaisOrcamento> itensAdicionaisOrcamentos;
	/**
	 * @return the data
	 */
	@DateTimeFormat
	@Temporal(TemporalType.DATE)
	@Required
	public Date getData() {
		return data;
	}
	/**
	 * @param data the data to set
	 */
	public void setData(Date data) {
		this.data = data;
	}
	/**
	 * @return the validade
	 */
	@Required
	@DisplayName("Validade(dia)")
	@MaxLength(4)
	public Short getValidade() {
		return validade;
	}
	/**
	 * @param validade the validade to set
	 */
	public void setValidade(Short validade) {
		this.validade = validade;
	}
	/**
	 * @return the desconto
	 */
	@Column(precision=6,scale=2)
	@NumberFormat(style = Style.PERCENT)
	@DisplayName("Desconto %")
	public BigDecimal getDesconto() {
		return desconto;
	}
	/**
	 * @param desconto the desconto to set
	 */
	public void setDesconto(BigDecimal desconto) {
		this.desconto = desconto;
	}
	/**
	 * @return the cliente
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@Required
	public Cliente getCliente() {
		return cliente;
	}
	/**
	 * @param cliente the cliente to set
	 */
	public void setCliente(Cliente cliente) {
		this.cliente = cliente;
	}
	/**
	 * @return the tiposAmostras
	 */
	@OneToMany(mappedBy = "orcamento",fetch=FetchType.LAZY)
	@Required
	public Set<OrcamentoTipoAmostra> getTiposAmostras() {
		return tiposAmostras;
	}
	/**
	 * @param tiposAmostras the tiposAmostras to set
	 */
	public void setTiposAmostras(Set<OrcamentoTipoAmostra> tiposAmostras) {
		this.tiposAmostras = tiposAmostras;
	}
	/**
	 * @return the valorTotal
	 */
	@Transient
	public Money getValorTotal() {
		return valorTotal;
	}
	/**
	 * @param valorTotal the valorTotal to set
	 */
	public void setValorTotal(Money valorTotal) {
		this.valorTotal = valorTotal;
	}
	/**
	 * @return the valorDesconto
	 */
	@Transient
	public Money getValorDesconto() {
		return ValorDesconto;
	}
	/**
	 * @param valorDesconto the valorDesconto to set
	 */
	public void setValorDesconto(Money valorDesconto) {
		ValorDesconto = valorDesconto;
	}
	/**
	 * @return the valorTotalLiq
	 */
	@Transient
	@DisplayName("Total Lquido")
	public Money getValorTotalLiq() {
		return valorTotalLiq;
	}
	/**
	 * @param valorTotalLiq the valorTotalLiq to set
	 */
	public void setValorTotalLiq(Money valorTotalLiq) {
		this.valorTotalLiq = valorTotalLiq;
	}
	/**
	 * @return the itensAdicionaisOrcamentos
	 */
	@OneToMany(mappedBy = "orcamento",fetch=FetchType.LAZY)
	@Required
	public Set<ItensAdicionaisOrcamento> getItensAdicionaisOrcamentos() {
		return itensAdicionaisOrcamentos;
	}
	/**
	 * @param itensAdicionaisOrcamentos the itensAdicionaisOrcamentos to set
	 */
	public void setItensAdicionaisOrcamentos(
			Set<ItensAdicionaisOrcamento> itensAdicionaisOrcamentos) {
		this.itensAdicionaisOrcamentos = itensAdicionaisOrcamentos;
	}
	/**
	 * @return the codigoId
	 */
	@Formula("id")
	@DisplayName("Id")
	@DescriptionProperty
	public Long getCodigoId() {
		return codigoId;
	}
	/**
	 * @param codigoId the codigoId to set
	 */
	public void setCodigoId(Long codigoId) {
		this.codigoId = codigoId;
	}

	
}
