package br.com.mss.labqp.orcamento.service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Set;

import org.nextframework.controller.crud.FiltroListagem;
import org.nextframework.persistence.ResultList;
import org.nextframework.service.GenericService;
import org.nextframework.types.Money;

import br.com.mss.labqp.orcamento.dao.OrcamentoDAO;
import br.com.mss.labqp.orcamento.entity.ItemOrcamento;
import br.com.mss.labqp.orcamento.entity.Orcamento;
import br.com.mss.labqp.orcamento.entity.OrcamentoTipoAmostra;

public class OrcamentoService extends GenericService<Orcamento> {

	ItemOrcamentoService itemOrcamentoService;
	OrcamentoDAO orcamentoDAO;
	
	/**
	 * @param orcamentoDAO the orcamentoDAO to set
	 */
	public void setOrcamentoDAO(OrcamentoDAO orcamentoDAO) {
		this.orcamentoDAO = orcamentoDAO;
	}

	/**
	 * @param itemOrcamentoService the itemOrcamentoService to set
	 */
	public void setItemOrcamentoService(ItemOrcamentoService itemOrcamentoService) {
		this.itemOrcamentoService = itemOrcamentoService;
	}

	/* (non-Javadoc)
	 * @see org.nextframework.service.GenericService#findForListagem(org.nextframework.controller.crud.FiltroListagem)
	 */
	@Override
	public ResultList<Orcamento> findForListagem(FiltroListagem filtro) {
		ResultList<Orcamento> rlOrc = super.findForListagem(filtro);
		if (rlOrc != null && !rlOrc.list().isEmpty()){
			for (Orcamento orcamento : rlOrc.list()) {
				Set<OrcamentoTipoAmostra> orctaList = orcamento.getTiposAmostras();
				Money[] totais = valorTotal(orctaList,orcamento.getDesconto());
				orcamento.setValorTotal(totais[0]);
				orcamento.setValorDesconto(totais[1]);
				orcamento.setValorTotalLiq(totais[2]);
			}
		}
		return rlOrc;
	}

	/* (non-Javadoc)
	 * @see org.nextframework.service.GenericService#loadForEntrada(java.lang.Object)
	 */
	@Override
	public Orcamento loadForEntrada(Orcamento bean) {
		bean = super.loadForEntrada(bean);
		Set<OrcamentoTipoAmostra> orctaList = bean.getTiposAmostras();
		Money[] totais = valorTotal(orctaList,bean.getDesconto());
		bean.setValorTotal(totais[0]);
		bean.setValorDesconto(totais[1]);
		bean.setValorTotalLiq(totais[2]);
		return bean;
	}

	private Money[] valorTotal(Set<OrcamentoTipoAmostra> orctaList,BigDecimal desconto ){
		Money[] vlrTotal = new Money[3];
		vlrTotal[0] = new Money("0");
		if (orctaList != null && !orctaList.isEmpty()) {
			for (OrcamentoTipoAmostra orcTA : orctaList) {
				Money valorTA = new Money("0");
				List<ItemOrcamento> ioList = itemOrcamentoService.findBy(orcTA,false,"");
				if (ioList != null && !ioList.isEmpty()) {
					for (ItemOrcamento io : ioList) {
						if (io.getValor()!= null)
							valorTA = valorTA.add(io.getValor());
					}
				}
				orcTA.setValorTotal(valorTA);
				vlrTotal[0] = vlrTotal[0].add(valorTA);
			}
		}
		Money vlrDesconto = new Money((desconto.divide(new BigDecimal(100.00))),2,RoundingMode.UP);
		vlrTotal[1] = vlrTotal[0].multiply( vlrDesconto);
		vlrTotal[2] = vlrTotal[0].subtract(vlrTotal[1]);
		return vlrTotal;
	}
	
}
