package br.com.mss.labqp.programacao.entity;

import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.nextframework.bean.annotation.DisplayName;
import org.nextframework.validation.annotation.MaxLength;
import org.nextframework.validation.annotation.MaxValue;
import org.nextframework.validation.annotation.Required;
import org.springframework.format.annotation.DateTimeFormat;

import br.com.mss.labqp.BasicBeanAudit;
import br.com.mss.labqp.geral.DiaSemanaEnum;
import br.com.mss.labqp.geral.FlagSimNaoEnum;
import br.com.mss.labqp.geral.TipoProgramacaoEnum;
import br.com.mss.labqp.orcamento.entity.Orcamento;

@Entity
@Table(name = "programacao")
@SequenceGenerator(allocationSize = 1, name = "SEQUENCE", sequenceName = "SQ_PROGRAMACAO_ID")
@DisplayName("Programao de Coleta")
public class Programacao extends BasicBeanAudit {

	/**
	 * 
	 */
	private static final long serialVersionUID = 170781901451599322L;
	private Orcamento orcamento;
	private Date dataInicial;
	private DiaSemanaEnum diaSemana;
	private Short periodicidade;
	private FlagSimNaoEnum horaMarcada;
	private TipoProgramacaoEnum tipoProgramacao;
	private String contato;
	private List<ServicoEsgotamento> servicosEsgotamento;
	private List<Kit> kits;

	@ManyToOne(fetch = FetchType.LAZY)
	public Orcamento getOrcamento() {
		return orcamento;
	}

	public void setOrcamento(Orcamento orcamento) {
		this.orcamento = orcamento;
	}

	/**
	 * @return the dataInicial
	 */
	@DateTimeFormat()
	@Temporal(TemporalType.DATE)
	@Required
	@DisplayName("Data inicial")
	public Date getDataInicial() {
		return dataInicial;
	}

	/**
	 * @param dataInicial the dataInicial to set
	 */
	public void setDataInicial(Date dataInicial) {
		this.dataInicial = dataInicial;
	}

	/**
	 * @return the diaSemana
	 */
	@DisplayName("Dia da semana")
	@Required
	public DiaSemanaEnum getDiaSemana() {
		return diaSemana;
	}

	/**
	 * @param diaSemana the diaSemana to set
	 */
	public void setDiaSemana(DiaSemanaEnum diaSemana) {
		this.diaSemana = diaSemana;
	}

	/**
	 * @return the periodicidade
	 */
	@Required
	@MaxLength(2)
	@MaxValue(99)
	public Short getPeriodicidade() {
		return periodicidade;
	}

	/**
	 * @param periodicidade the periodicidade to set
	 */
	public void setPeriodicidade(Short periodicidade) {
		this.periodicidade = periodicidade;
	}

	/**
	 * @return the tipoProgramacao
	 */
	@Required
	@DisplayName("Tipo de programao")
	public TipoProgramacaoEnum getTipoProgramacao() {
		return tipoProgramacao;
	}

	/**
	 * @param tipoProgramacao the tipoProgramacao to set
	 */
	public void setTipoProgramacao(TipoProgramacaoEnum tipoProgramacao) {
		this.tipoProgramacao = tipoProgramacao;
	}

	/**
	 * @return the ponto
	 */
	/**
	 * @return the horaMarcada
	 */
	@DisplayName("Hora marcada?")
	@Required
	public FlagSimNaoEnum getHoraMarcada() {
		return horaMarcada;
	}

	/**
	 * @param horaMarcada the horaMarcada to set
	 */
	public void setHoraMarcada(FlagSimNaoEnum horaMarcada) {
		this.horaMarcada = horaMarcada;
	}

	/**
	 * @return the contato
	 */
	@MaxLength(40)
	@Column(length = 40)
	public String getContato() {
		return contato;
	}

	/**
	 * @param contato the contato to set
	 */
	public void setContato(String contato) {
		this.contato = contato;
	}

	/**
	 * @return the servicosEsgotamento
	 */
	@OneToMany(mappedBy = "programacao")
	@DisplayName("Servio de Esgotamento")
	public List<ServicoEsgotamento> getServicosEsgotamento() {
		return servicosEsgotamento;
	}

	/**
	 * @param servicosEsgotamento the servicosEsgotamento to set
	 */
	public void setServicosEsgotamento(List<ServicoEsgotamento> servicosEsgotamento) {
		this.servicosEsgotamento = servicosEsgotamento;
	}

	/**
	 * @return the kits
	 */
	@OneToMany(mappedBy = "programacao")
	@DisplayName("Kit")
	public List<Kit> getKits() {
		return kits;
	}

	/**
	 * @param kits the kits to set
	 */
	public void setKits(List<Kit> kits) {
		this.kits = kits;
	}

}
