package br.com.mss.labqp.requisicao.dao;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Query;
import org.nextframework.controller.crud.FiltroListagem;
import org.nextframework.persistence.QueryBuilder;
import org.nextframework.persistence.SaveOrUpdateStrategy;
import org.nextframework.types.Money;

import br.com.mss.labqp.GenericAuditDAO;
import br.com.mss.labqp.cadastro.dao.ClienteDAO;
import br.com.mss.labqp.cadastro.dao.TipoAmostraDAO;
import br.com.mss.labqp.cadastro.entity.Cliente;
import br.com.mss.labqp.cadastro.entity.TipoAmostra;
import br.com.mss.labqp.geral.FiltroExcluidoEnum;
import br.com.mss.labqp.geral.FlagSimNaoEnum;
import br.com.mss.labqp.geral.ImpostoEnum;
import br.com.mss.labqp.geral.SituacaoDocEnum;
import br.com.mss.labqp.requisicao.entity.ExtratoMensal;
import br.com.mss.labqp.requisicao.entity.EnsaioAnaliseReport;
import br.com.mss.labqp.requisicao.entity.QtdeRequisicao;
import br.com.mss.labqp.requisicao.entity.RequisicaoAnalise;
import br.com.mss.labqp.requisicao.entity.RequisicaoAnaliseFiltro;

public class RequisicaoAnaliseDAO extends GenericAuditDAO<RequisicaoAnalise> {

	private ClienteDAO clienteDAO;
	private TipoAmostraDAO tipoAmostraDAO;
	
	/**
	 * @param clienteDAO the clienteDAO to set
	 */
	public void setClienteDAO(ClienteDAO clienteDAO) {
		this.clienteDAO = clienteDAO;
	}

	/**
	 * @param tipoAmostraDAO the tipoAmostraDAO to set
	 */
	public void setTipoAmostraDAO(TipoAmostraDAO tipoAmostraDAO) {
		this.tipoAmostraDAO = tipoAmostraDAO;
	}

	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateListagemQuery(org.nextframework.persistence.QueryBuilder, org.nextframework.controller.crud.FiltroListagem)
	 */
	@Override
	public void updateListagemQuery(QueryBuilder<RequisicaoAnalise> query,
			FiltroListagem _filtro) {
		query = montaQuery(query, (RequisicaoAnaliseFiltro)_filtro);
		
	}

	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateSaveOrUpdate(org.nextframework.persistence.SaveOrUpdateStrategy)
	 */
	@Override
	public void updateSaveOrUpdate(SaveOrUpdateStrategy save) {
		save.saveOrUpdateManaged("itensRequisicaoAnalise");
		super.updateSaveOrUpdate(save);
	}
	
	/* (non-Javadoc)
	 * @see org.nextframework.persistence.GenericDAO#updateEntradaQuery(org.nextframework.persistence.QueryBuilder)
	 */
	@Override
	public void updateEntradaQuery(QueryBuilder<RequisicaoAnalise> query) {
		query.fetchCollection("itensRequisicaoAnalise");
		query.joinFetch("requisicaoAnalise.cliente");
		query.leftOuterJoinFetch("requisicaoAnalise.tipoAmostra");
		query.leftOuterJoinFetch("requisicaoAnalise.coletador");
	
		super.updateEntradaQuery(query);
	}
	
	public List<RequisicaoAnalise> findByFiltro(RequisicaoAnaliseFiltro filtro){
		QueryBuilder<RequisicaoAnalise> query = montaQuery(this.query(), filtro);
		return query.list();
	}
	
	public List<ExtratoMensal> extratoRequisicao(RequisicaoAnaliseFiltro filtro){
        

		StringBuilder queryString = new StringBuilder();
		queryString.append("select req.tipoAmostra.id, req.tipoAmostra.nome, sum(idoc.valor), count(idoc.id)");
		queryString.append(" from ItemDocumento as idoc, RequisicaoAnalise as req ");
		queryString.append(" where idoc.idDoc = req.id ");	
		queryString.append(" and idoc.documento.ano = :ano");	
		queryString.append(" and idoc.documento.mes = :mes");	
		queryString.append(" and idoc.documento.dataProcessamento between :d1 and :d2");
		queryString.append(" and idoc.documento.cliente = :cliente");
		queryString.append(" and idoc.documento.situacaoDocEnum = :situacao");
		queryString.append(" and idoc.documento.dataExc is null");
		queryString.append(" group by req.tipoAmostra.id, req.tipoAmostra.nome ");
		
	
		Query select = getSession().createQuery(queryString.toString());
		select.setParameter("ano", filtro.getAno());
		select.setParameter("mes", filtro.getMes());
		select.setParameter("d1", filtro.getDataI());
		select.setParameter("d2", filtro.getDataF());
		select.setParameter("cliente", filtro.getCliente());
		select.setParameter("situacao", SituacaoDocEnum.ABERTO);
       
		List<ExtratoMensal> extratoMensalList = new ArrayList<ExtratoMensal>();
		@SuppressWarnings("unchecked")
		List<Object[]> lista = select.list();
		ExtratoMensal extratoMensal;
		Long taId;
		for (Object[] object : lista) {
			extratoMensal = new ExtratoMensal();
			taId = (Long) object[0];
			extratoMensal.setTaId(taId);
			extratoMensal.setTaNome(object[1].toString());
			extratoMensal.setTotal( new Money((Long) object[2],true) );
			extratoMensal.setQuantidade(new Integer(object[3].toString()));
			extratoMensal.setListEnsaioAnalise(parametrosPorTipoAmostra(filtro,taId));
			extratoMensalList.add(extratoMensal);
		}

		return extratoMensalList;		
	}

	/**
	 * 
	 */
	@SuppressWarnings("unchecked")
	public List<Long> documentosExtrato(RequisicaoAnaliseFiltro filtro) {
		StringBuilder queryString = new StringBuilder();
		queryString.append("select distinct req.id ");
		queryString.append(" from ItemDocumento as idoc, RequisicaoAnalise as req ");
		queryString.append(" where idoc.idDoc = req.id ");	
		queryString.append(" and idoc.documento.ano = :ano");	
		queryString.append(" and idoc.documento.mes = :mes");	
		queryString.append(" and idoc.documento.dataProcessamento between :d1 and :d2");
		queryString.append(" and idoc.documento.cliente = :cliente");
		queryString.append(" and idoc.documento.dataExc is null");
		queryString.append(" and idoc.documento.situacaoDocEnum = :situacao");
		
		Query select = getSession().createQuery(queryString.toString());
		select.setParameter("ano", filtro.getAno());
		select.setParameter("mes", filtro.getMes());
		select.setParameter("d1", filtro.getDataI());
		select.setParameter("d2", filtro.getDataF());
		select.setParameter("cliente", filtro.getCliente());
		select.setParameter("situacao", SituacaoDocEnum.ABERTO);

		return select.list();
	}
	
	/**
	 * 
	 */
	@SuppressWarnings("unchecked")
	private List<EnsaioAnaliseReport> parametrosPorTipoAmostra(RequisicaoAnaliseFiltro filtro, Long tipoAmostraId) {
		StringBuilder queryString = new StringBuilder();
		queryString.append("select distinct iReq.parametroAnalise.id, iReq.parametroAnalise.nome ");
		queryString.append(" from ItemRequisicaoAnalise as iReq, ItemDocumento as idoc, Documento as doc ");
		queryString.append(" where idoc.idDoc = iReq.requisicaoAnalise ");	
		queryString.append(" and iReq.requisicaoAnalise.tipoAmostra.id = :tipoAmostraId");	
		queryString.append(" and doc.id = idoc.documento.id");	
		queryString.append(" and doc.ano = :ano");	
		queryString.append(" and doc.mes = :mes");	
		queryString.append(" and doc.dataProcessamento between :d1 and :d2");
		queryString.append(" and doc.cliente = :cliente");
		queryString.append(" and doc.dataExc is null");
		queryString.append(" and doc.situacaoDocEnum = :situacao");
		
		Query select = getSession().createQuery(queryString.toString());
		select.setParameter("tipoAmostraId", tipoAmostraId);
		select.setParameter("ano", filtro.getAno());
		select.setParameter("mes", filtro.getMes());
		select.setParameter("d1", filtro.getDataI());
		select.setParameter("d2", filtro.getDataF());
		select.setParameter("cliente", filtro.getCliente());
		select.setParameter("situacao", SituacaoDocEnum.ABERTO);

		List<EnsaioAnaliseReport> paList = new ArrayList<EnsaioAnaliseReport>();
		List<Object[]> lista = select.list();
		EnsaioAnaliseReport ensaioAnaliseReport;
		for (Object[] object : lista) {
			ensaioAnaliseReport = new EnsaioAnaliseReport();
			ensaioAnaliseReport.setPaId((Long) object[0]);
			ensaioAnaliseReport.setPaNome(object[1].toString());
			paList.add(ensaioAnaliseReport);
		}

		return paList;		
	}
	
	private QueryBuilder<RequisicaoAnalise> montaQuery(QueryBuilder<RequisicaoAnalise> query, RequisicaoAnaliseFiltro filtro) {
		query.joinFetch("requisicaoAnalise.cliente");
		query.joinFetch("requisicaoAnalise.tipoAmostra");
		query.joinFetch("requisicaoAnalise.orcamentoTipoAmostra");
		query.leftOuterJoinFetch("requisicaoAnalise.coletador");
		query.where("requisicaoAnalise.id = ?", filtro.getId());
		query.where("requisicaoAnalise.cliente = ? ", filtro.getCliente());
		query.where("requisicaoAnalise.tipoAmostra = ? ", filtro.getTipoAmostra());
		query.where("requisicaoAnalise.coletador = ? ", filtro.getColetador());
		query.where("requisicaoAnalise.situacaoRequisicaoEnum = ? ", filtro.getSituacaoRequisicaoEnum());
		query.whereLike("requisicaoAnalise.indentificacaoAmostra", filtro.getIdentificaoAmostra());
		query.whereLike("requisicaoAnalise.localColeta", filtro.getLocalColeta());
		if (filtro.getFiltroExcluido() != null) {
			if (filtro.getFiltroExcluido() == FiltroExcluidoEnum.SIM)
				query.where("requisicaoAnalise.dataExc is not null");
			else
				query.where("requisicaoAnalise.dataExc is null");
		}	
		return query;	
	}

	public List<ExtratoMensal> extratoItensAdicionais(RequisicaoAnaliseFiltro filtro){
	    
		StringBuilder queryString = new StringBuilder();
		queryString.append("select idoc.descricao, sum(idoc.valor), count(idoc.id)");
		queryString.append(" from ItemDocumento as idoc ");
		queryString.append(" where idoc.tipoDocumento.eRequisicao = :eRequisicao");	
		queryString.append(" and idoc.documento.ano = :ano");	
		queryString.append(" and idoc.documento.mes = :mes");	
		queryString.append(" and idoc.documento.dataProcessamento between :d1 and :d2");
		queryString.append(" and idoc.documento.cliente = :cliente");
		queryString.append(" and idoc.documento.dataExc is null");
		queryString.append(" and idoc.documento.situacaoDocEnum = :situacao");
		queryString.append(" group by idoc.descricao ");
		
	
		Query select = getSession().createQuery(queryString.toString());
		select.setParameter("eRequisicao", FlagSimNaoEnum.NAO);
		select.setParameter("ano", filtro.getAno());
		select.setParameter("mes", filtro.getMes());
		select.setParameter("d1", filtro.getDataI());
		select.setParameter("d2", filtro.getDataF());
		select.setParameter("cliente", filtro.getCliente());
		select.setParameter("situacao", SituacaoDocEnum.ABERTO);
	
		List<ExtratoMensal> extratoAdicList = new ArrayList<ExtratoMensal>();
		@SuppressWarnings("unchecked")
		List<Object[]> lista = select.list();
		ExtratoMensal extratoMensal;
		for (Object[] object : lista) {
			extratoMensal = new ExtratoMensal();
			extratoMensal.setTaNome(object[0].toString());
			extratoMensal.setTotal( new Money((Long) object[1],true) );
			extratoMensal.setQuantidade(new Integer(object[2].toString()));
			extratoAdicList.add(extratoMensal);
		}
	
		return extratoAdicList;		
	}

	public List<ExtratoMensal> extratoImpostos(RequisicaoAnaliseFiltro filtro){
	    
		StringBuilder queryString = new StringBuilder();
		queryString.append("select idoc.imposto, sum(idoc.valor)");
		queryString.append(" from ImpostoDocumento as idoc ");
		queryString.append(" where idoc.documento.ano = :ano");	
		queryString.append(" and idoc.documento.mes = :mes");	
		queryString.append(" and idoc.documento.dataProcessamento between :d1 and :d2");
		queryString.append(" and idoc.documento.cliente = :cliente");
		queryString.append(" and idoc.documento.dataExc is null");
		queryString.append(" and idoc.documento.situacaoDocEnum = :situacao");
		queryString.append(" group by idoc.imposto ");
		
	
		Query select = getSession().createQuery(queryString.toString());
		select.setParameter("ano", filtro.getAno());
		select.setParameter("mes", filtro.getMes());
		select.setParameter("d1", filtro.getDataI());
		select.setParameter("d2", filtro.getDataF());
		select.setParameter("cliente", filtro.getCliente());
		select.setParameter("situacao", SituacaoDocEnum.ABERTO);
	
		List<ExtratoMensal> extratoImpList = new ArrayList<ExtratoMensal>();
		@SuppressWarnings("unchecked")
		List<Object[]> lista = select.list();
		ExtratoMensal extratoMensal;
		for (Object[] object : lista) {
			extratoMensal = new ExtratoMensal();
			extratoMensal.setTaNome(((ImpostoEnum) object[0]).toString());
			extratoMensal.setTotal( new Money((Long) object[1],true) );
			extratoImpList.add(extratoMensal);
		}
	
		return extratoImpList;		
	}	

	public List<QtdeRequisicao> qtdeRequisicao(RequisicaoAnaliseFiltro filtro){
		StringBuilder queryString = new StringBuilder();
		queryString.append("select req.cliente.id, req.tipoAmostra.id, count(req.tipoAmostra)");
		queryString.append(" from RequisicaoAnalise as req ");
		queryString.append(" where req.dataEntrada between :d1 and :d2");	
		queryString.append(" and req.cliente = coalesce(:cliente,req.cliente)");
		queryString.append(" and req.tipoAmostra = coalesce(:tipoAmostra,req.tipoAmostra)");
		queryString.append(" and req.situacaoRequisicaoEnum = coalesce(:situacao,req.situacaoRequisicaoEnum)");
		queryString.append(" and req.dataExc is null");
		queryString.append(" group by req.cliente.id, req.tipoAmostra.id ");
		
	
		Query select = getSession().createQuery(queryString.toString());
		select.setParameter("d1", filtro.getDataI());
		select.setParameter("d2", filtro.getDataF());
		select.setParameter("cliente", filtro.getCliente());
		select.setParameter("situacao", filtro.getSituacaoRequisicaoEnum());
		select.setParameter("tipoAmostra", filtro.getTipoAmostra());
	
		List<QtdeRequisicao> qtdeReqList = new ArrayList<QtdeRequisicao>();
		@SuppressWarnings("unchecked")
		List<Object[]> lista = select.list();
		QtdeRequisicao qtdeRequisicao;
		Cliente cliente;
		TipoAmostra tipoAmostra;
		Integer qtde;
		for (Object[] object : lista) {
			
			cliente = new Cliente();
			cliente.setId((Long) object[0]);
			cliente = clienteDAO.load(cliente);
			
			tipoAmostra = new TipoAmostra();
			tipoAmostra.setId((Long) object[1]);
			tipoAmostra = tipoAmostraDAO.load(tipoAmostra);
			
			qtde = ((Long) object[2]).intValue();
			
			
			qtdeRequisicao = new QtdeRequisicao();
			qtdeRequisicao.setCliente(cliente);
			qtdeRequisicao.setTipoAmostra(tipoAmostra);
			qtdeRequisicao.setQuantidade( qtde );
			qtdeReqList.add(qtdeRequisicao);
		}
	
		return qtdeReqList;		
	}

}
