package br.com.mss.labqp.requisicao.entity;

import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Formula;
import org.nextframework.bean.annotation.DescriptionProperty;
import org.nextframework.bean.annotation.DisplayName;
import org.nextframework.validation.annotation.MaxLength;
import org.nextframework.validation.annotation.Required;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.format.annotation.DateTimeFormat.ISO;

import br.com.mss.labqp.BasicBeanAudit;
import br.com.mss.labqp.cadastro.entity.Cliente;
import br.com.mss.labqp.cadastro.entity.Coletador;
import br.com.mss.labqp.cadastro.entity.TipoAmostra;
import br.com.mss.labqp.geral.SituacaoRequisicaoEnum;
import br.com.mss.labqp.orcamento.entity.Orcamento;
import br.com.mss.labqp.orcamento.entity.OrcamentoTipoAmostra;


@Entity
@Table(name = "requisicaoAnalise")
@DisplayName("Requisio de Anlise")
@SequenceGenerator(allocationSize = 1, name = "SEQUENCE", sequenceName = "SQ_REQANALISE_ID")
public class RequisicaoAnalise extends BasicBeanAudit{

	/**
	 * 
	 */
	private static final long serialVersionUID = -4132200998459548197L;
	
	private Coletador coletador;
	private Date dataColeta;
	private Date horaColeta;
	private Date dataEntrada;
	private Date horaEntrada;
	private Cliente cliente;
	private TipoAmostra tipoAmostra;
	private String identificaoAmostra;
	private String cispoa;
	private Date dataSaida;
	private String localColeta;
	private Date dataFabricacao;
	private Date dataValidade;
	private String temperaturaAmostra;
	private String quantidadeAmostra;
	private String embalagem;
	private String lacreCispoa;
	private String conclusao;
	private SituacaoRequisicaoEnum situacaoRequisicaoEnum;
	private Orcamento orcamento;
	private OrcamentoTipoAmostra orcamentoTipoAmostra;
	private String latitude;
	private String longitude;
	private String temperaturaAr;
	private String condicaoTempo;
	private String identGPS;
	private Long codigoId; // Formula - usado como DescriptionProperty
	
	private List<ItemRequisicaoAnalise> itensRequisicaoAnalise;
	
	private Boolean selecionado;

	
	/**
	 * @return the coletador
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@Required
	public Coletador getColetador() {
		return coletador;
	}
	/**
	 * @param coletador the coletador to set
	 */
	public void setColetador(Coletador coletador) {
		this.coletador = coletador;
	}
	/**
	 * @return the datahoraColeta
	 */
	@DateTimeFormat(iso=ISO.DATE, pattern="dd/MM/yyyy")
	@Temporal(TemporalType.DATE)
	public Date getDataColeta() {
		return dataColeta;
	}
	/**
	 * @param datahoraColeta the datahoraColeta to set
	 */
	public void setDataColeta(Date dataColeta) {
		this.dataColeta = dataColeta;
	}
	/**
	 * @return the dataEntrada
	 */
	@DateTimeFormat(iso=ISO.DATE, pattern="dd/MM/yyyy")
	@Temporal(TemporalType.DATE)
	public Date getDataEntrada() {
		return dataEntrada;
	}
	/**
	 * @param dataEntrada the dataEntrada to set
	 */
	public void setDataEntrada(Date dataEntrada) {
		this.dataEntrada = dataEntrada;
	}
	/**
	 * @return the tipoAmostra
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@Required
	public TipoAmostra getTipoAmostra() {
		return tipoAmostra;
	}
	/**
	 * @param tipoAmostra the tipoAmostra to set
	 */
	public void setTipoAmostra(TipoAmostra tipoAmostra) {
		this.tipoAmostra = tipoAmostra;
	}
	/**
	 * @return the identificaoAmostra
	 */
	@DisplayName("Identificao da Amostra")
	@Column(length = 100)
	@MaxLength(100)
	public String getIdentificaoAmostra() {
		return identificaoAmostra;
	}
	/**
	 * @param identificaoAmostra the identificaoAmostra to set
	 */
	public void setIdentificaoAmostra(String identificaoAmostra) {
		this.identificaoAmostra = identificaoAmostra;
	}
	/**
	 * @return the dataSaida
	 */
	@DateTimeFormat
	@Temporal(TemporalType.DATE)
	public Date getDataSaida() {
		return dataSaida;
	}
	/**
	 * @param dataSaida the dataSaida to set
	 */
	public void setDataSaida(Date dataSaida) {
		this.dataSaida = dataSaida;
	}
	/**
	 * @return the localColeta
	 */
	@DisplayName("Local da Coleta")
	@Column(length = 50)
	@MaxLength(50)
	public String getLocalColeta() {
		return localColeta;
	}
	/**
	 * @param localColeta the localColeta to set
	 */
	public void setLocalColeta(String localColeta) {
		this.localColeta = localColeta;
	}
	/**
	 * @return the cliente
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@Required
	public Cliente getCliente() {
		return cliente;
	}
	/**
	 * @param cliente the cliente to set
	 */
	public void setCliente(Cliente cliente) {
		this.cliente = cliente;
	}
	/**
	 * @return the itensRequisicaoAnalise
	 */
	@OneToMany(mappedBy = "requisicaoAnalise")
	@DisplayName("Itens Requisio Anlise")
	public List<ItemRequisicaoAnalise> getItensRequisicaoAnalise() {
		return itensRequisicaoAnalise;
	}
	/**
	 * @param itensRequisicaoAnalise the itensRequisicaoAnalise to set
	 */
	public void setItensRequisicaoAnalise(
			List<ItemRequisicaoAnalise> itensRequisicaoAnalise) {
		this.itensRequisicaoAnalise = itensRequisicaoAnalise;
	}
	/**
	 * @return the horaColeta
	 */
	@DateTimeFormat(iso=ISO.TIME, pattern="HH:mm")
	@Temporal(TemporalType.TIME)
	public Date getHoraColeta() {
		return horaColeta;
	}
	/**
	 * @param horaColeta the horaColeta to set
	 */
	public void setHoraColeta(Date horaColeta) {
		this.horaColeta = horaColeta;
	}
	/**
	 * @return the horaEntrada
	 */
	@DateTimeFormat(iso=ISO.TIME, pattern="HH:mm")
	@Temporal(TemporalType.TIME)
	public Date getHoraEntrada() {
		return horaEntrada;
	}
	/**
	 * @param horaEntrada the horaEntrada to set
	 */
	public void setHoraEntrada(Date horaEntrada) {
		this.horaEntrada = horaEntrada;
	}
	/**
	 * @return the cispoa
	 */
	@Column(length = 20)
	@MaxLength(20)
	@DisplayName("CISPOA")
	public String getCispoa() {
		return cispoa;
	}
	/**
	 * @param cispoa the cispoa to set
	 */
	public void setCispoa(String cispoa) {
		this.cispoa = cispoa;
	}
	/**
	 * @return the dataFabricacao
	 */
	@DateTimeFormat(iso=ISO.DATE, pattern="dd/MM/yyyy")
	@Temporal(TemporalType.DATE)
	@DisplayName("Data Fabricao")
	public Date getDataFabricacao() {
		return dataFabricacao;
	}
	/**
	 * @param dataFabricacao the dataFabricacao to set
	 */
	public void setDataFabricacao(Date dataFabricacao) {
		this.dataFabricacao = dataFabricacao;
	}
	/**
	 * @return the dataValidade
	 */
	@DateTimeFormat(iso=ISO.DATE, pattern="dd/MM/yyyy")
	@Temporal(TemporalType.DATE)
	public Date getDataValidade() {
		return dataValidade;
	}
	/**
	 * @param dataValidade the dataValidade to set
	 */
	public void setDataValidade(Date dataValidade) {
		this.dataValidade = dataValidade;
	}
	/**
	 * @return the temperaturaAmostra
	 */
	@Column(length = 50)
	@MaxLength(50)
	public String getTemperaturaAmostra() {
		return temperaturaAmostra;
	}
	/**
	 * @param temperaturaAmostra the temperaturaAmostra to set
	 */
	public void setTemperaturaAmostra(String temperaturaAmostra) {
		this.temperaturaAmostra = temperaturaAmostra;
	}
	/**
	 * @return the quantidadeAmostra
	 */
	@Column(length = 50)
	@MaxLength(50)
	public String getQuantidadeAmostra() {
		return quantidadeAmostra;
	}
	/**
	 * @param quantidadeAmostra the quantidadeAmostra to set
	 */
	public void setQuantidadeAmostra(String quantidadeAmostra) {
		this.quantidadeAmostra = quantidadeAmostra;
	}
	/**
	 * @return the embalagem
	 */
	@Column(length = 50)
	@MaxLength(50)
	public String getEmbalagem() {
		return embalagem;
	}
	/**
	 * @param embalagem the embalagem to set
	 */
	public void setEmbalagem(String embalagem) {
		this.embalagem = embalagem;
	}
	/**
	 * @return the lacreCispoa
	 */
	@Column(length = 50)
	@MaxLength(50)
	@DisplayName("Lacre CISPOA")
	public String getLacreCispoa() {
		return lacreCispoa;
	}
	/**
	 * @param lacreCispoa the lacreCispoa to set
	 */
	public void setLacreCispoa(String lacreCispoa) {
		this.lacreCispoa = lacreCispoa;
	}
	/**
	 * @return the conclusao
	 */
	@Column(length = 150)
	@MaxLength(150)
	@DisplayName("Concluso")
	public String getConclusao() {
		return conclusao;
	}
	/**
	 * @param conclusao the conclusao to set
	 */
	public void setConclusao(String conclusao) {
		this.conclusao = conclusao;
	}
	/**
	 * @return the situacaoRequisicaoEnum
	 */
	@DisplayName("Situao")
	public SituacaoRequisicaoEnum getSituacaoRequisicaoEnum() {
		return situacaoRequisicaoEnum;
	}
	/**
	 * @param situacaoRequisicaoEnum the situacaoRequisicaoEnum to set
	 */
	public void setSituacaoRequisicaoEnum(
			SituacaoRequisicaoEnum situacaoRequisicaoEnum) {
		this.situacaoRequisicaoEnum = situacaoRequisicaoEnum;
	}
	/**
	 * @return the orcamento
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@DisplayName("Oramento")
	@Required
	public Orcamento getOrcamento() {
		return orcamento;
	}
	/**
	 * @param orcamento the orcamento to set
	 */
	public void setOrcamento(Orcamento orcamento) {
		this.orcamento = orcamento;
	}
	/**
	 * @return the codigoId
	 */
	@Formula("id")
	@DisplayName("Id")
	@DescriptionProperty
	public Long getCodigoId() {
		return codigoId;
	}
	/**
	 * @param codigoId the codigoId to set
	 */
	public void setCodigoId(Long codigoId) {
		this.codigoId = codigoId;
	}
	/**
	 * @return the selecionado
	 */
	@Transient
	public Boolean getSelecionado() {
		return selecionado;
	}
	/**
	 * @param selecionado the selecionado to set
	 */
	public void setSelecionado(Boolean selecionado) {
		this.selecionado = selecionado;
	}
	/**
	 * @return the latitude
	 */
	@Column(length = 20)
	@MaxLength(20)
	@DisplayName("Latitude(S)")
	public String getLatitude() {
		return latitude;
	}
	/**
	 * @param latitude the latitude to set
	 */
	public void setLatitude(String latitude) {
		this.latitude = latitude;
	}
	/**
	 * @return the longitude
	 */
	@Column(length = 20)
	@MaxLength(20)
	@DisplayName("Longitude(W)")
	public String getLongitude() {
		return longitude;
	}
	/**
	 * @param longitude the longitude to set
	 */
	public void setLongitude(String longitude) {
		this.longitude = longitude;
	}
	/**
	 * @return the temperaturaAr
	 */
	@Column(length = 20)
	@MaxLength(20)
	@DisplayName("Temperatura do Ar")
	public String getTemperaturaAr() {
		return temperaturaAr;
	}
	/**
	 * @param temperaturaAr the temperaturaAr to set
	 */
	public void setTemperaturaAr(String temperaturaAr) {
		this.temperaturaAr = temperaturaAr;
	}
	/**
	 * @return the condicaoTempo
	 */
	@Column(length = 20)
	@MaxLength(20)
	@DisplayName("Condio do Tempo")
	public String getCondicaoTempo() {
		return condicaoTempo;
	}
	/**
	 * @param condicaoTempo the condicaoTempo to set
	 */
	public void setCondicaoTempo(String condicaoTempo) {
		this.condicaoTempo = condicaoTempo;
	}
	/**
	 * @return the identGPS
	 */
	@Column(length = 20)
	@MaxLength(20)
	@DisplayName("Ident. GPS")
	public String getIdentGPS() {
		return identGPS;
	}
	/**
	 * @param identGPS the identGPS to set
	 */
	public void setIdentGPS(String identGPS) {
		this.identGPS = identGPS;
	}
	/**
	 * @return the orcamentoTipoAmostra
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@DisplayName("Tipo de Amostra/Ponto Coleta")
	@Required
	public OrcamentoTipoAmostra getOrcamentoTipoAmostra() {
		return orcamentoTipoAmostra;
	}
	/**
	 * @param orcamentoTipoAmostra the orcamentoTipoAmostra to set
	 */
	public void setOrcamentoTipoAmostra(OrcamentoTipoAmostra orcamentoTipoAmostra) {
		this.orcamentoTipoAmostra = orcamentoTipoAmostra;
	}
}
